'''
    MFEM Example 18 - Serial/Parallel Shared Code

    This is a python translation of ex18.hpp

    note: following variabls are set from ex18 or ex18p
        problem    
        num_equation
        max_char_speed 
        specific_heat_ratio;
        gas_constant;

'''
import numpy as np
from mfem import mfem_mode

if mfem_mode is None or mfem_mode == 'serial':
    import mfem.ser as mfem
else:
    import mfem.par as mfem    

num_equation = 0
specific_heat_ratio = 0
gas_constant = 0
problem = 0
max_char_speed = 0

class FE_Evolution(mfem.TimeDependentOperator):
    def __init__(self, vfes, A, A_flux):
        self.dim = vfes.GetFE(0).GetDim()
        self.vfes = vfes
        self.A = A
        self.Aflux = A_flux
        self.Me_inv = mfem.DenseTensor(vfes.GetFE(0).GetDof(),
                                       vfes.GetFE(0).GetDof(),
                                       vfes.GetNE())

        self.state = mfem.Vector(num_equation)
        self.f = mfem.DenseMatrix(num_equation, self.dim)
        self.flux = mfem.DenseTensor(vfes.GetNDofs(), self.dim, num_equation)
        self.z = mfem.Vector(A.Height())
        
        dof = vfes.GetFE(0).GetDof()
        Me = mfem.DenseMatrix(dof)
        inv = mfem.DenseMatrixInverse(Me)
        mi = mfem.MassIntegrator()
        for i in range(vfes.GetNE()):
            mi.AssembleElementMatrix(vfes.GetFE(i), vfes.GetElementTransformation(i), Me)
            inv.Factor()
            inv.GetInverseMatrix(self.Me_inv(i))
        super(FE_Evolution, self).__init__(A.Height())            
            
    def GetFlux(self, x, flux):
        state = self.state
        dof = self.flux.SizeI();
        dim = self.flux.SizeJ();
        for i in range(dof):
            for k in range(num_equation):
                self.state[k] = x[i, k]
            ComputeFlux(state, dim, self.f)
            for d in range(dim):
                for k in range(num_equation):                
                    flux[i, d, k] = self.f[k, d]
        mcs = ComputeMaxCharSpeed(state, dim)
        if (mcs > max_char_speed):
           globals()['max_char_speed'] = mcs
        
    def Mult(self, x, y):
        globals()['max_char_speed'] = 0.;
        num_equation = globals()['num_equation']
        # 1. Create the vector z with the face terms -<F.n(u), [w]>.
        self.A.Mult(x, self.z);

        # 2. Add the element terms.
        # i.  computing the flux approximately as a grid function by interpolating
        #     at the solution nodes.
        # ii. multiplying this grid function by a (constant) mixed bilinear form for
        #     each of the num_equation, computing (F(u), grad(w)) for each equation.

        xmat = mfem.DenseMatrix(x.GetData(), self.vfes.GetNDofs(), num_equation)
        self.GetFlux(xmat, self.flux)

        for k in range(num_equation):
           fk = mfem.Vector(self.flux[k].GetData(), self.dim * self.vfes.GetNDofs())
           o = k * self.vfes.GetNDofs()
           zk = self.z[o: o+self.vfes.GetNDofs()]
           self.Aflux.AddMult(fk, zk)

        # 3. Multiply element-wise by the inverse mass matrices.
        zval = mfem.Vector()
        vdofs = mfem.intArray()
        dof = self.vfes.GetFE(0).GetDof()
        zmat = mfem.DenseMatrix()
        ymat = mfem.DenseMatrix(dof, num_equation)        

        for i in range(self.vfes.GetNE()):
            # Return the vdofs ordered byNODES
            vdofs = mfem.intArray(self.vfes.GetElementVDofs(i))
            self.z.GetSubVector(vdofs, zval)
            zmat.UseExternalData(zval.GetData(), dof, num_equation)
            mfem.Mult(self.Me_inv[i], zmat, ymat);
            y.SetSubVector(vdofs, ymat.GetData())
        
class DomainIntegrator(mfem.BilinearFormIntegrator):
    def __init__(self, dim):
        num_equation = globals()['num_equation']
        self.flux= mfem.DenseMatrix(num_equation, dim)        
        self.shape = mfem.Vector()
        self.dshapedr = mfem.DenseMatrix()
        self.dshapedx = mfem.DenseMatrix()
        super(DomainIntegrator, self).__init__()
        
    def AssembleElementMatrix2(self, trial_fe, test_fe, Tr, elmat):
        # Assemble the form (vec(v), grad(w))

        # Trial space = vector L2 space (mesh dim)
        # Test space  = scalar L2 space

        dof_trial = trial_fe.GetDof()
        dof_test = test_fe.GetDof()
        dim = trial_fe.GetDim()

        self.shape.SetSize(dof_trial)
        self.dshapedr.SetSize(dof_test, dim)
        self.dshapedx.SetSize(dof_test, dim)

        elmat.SetSize(dof_test, dof_trial * dim);
        elmat.Assign(0.0)

        maxorder = max(trial_fe.GetOrder(), test_fe.GetOrder());
        intorder = 2 * maxorder;
        ir = mfem.IntRules.Get(trial_fe.GetGeomType(), intorder)

        for i in range(ir.GetNPoints()):
            ip = ir.IntPoint(i)
            
            # Calculate the shape functions
            trial_fe.CalcShape(ip, self.shape);
            self.shape *= ip.weight
            
            # Compute the physical gradients of the test functions
            Tr.SetIntPoint(ip)
            test_fe.CalcDShape(ip, self.dshapedr)
            mfem.Mult(self.dshapedr, Tr.AdjugateJacobian(), self.dshapedx)

            for d in range(dim):
                for j in range(dof_test):
                    for k in range(dof_trial):
                       elmat[j, k + d * dof_trial] += self.shape[k] * self.dshapedx[j, d]

class FaceIntegrator(mfem.NonlinearFormIntegrator):
    def __init__(self, rsolver, dim):
        self.rsolver = rsolver
        self.shape1 = mfem.Vector()
        self.shape2 = mfem.Vector()
        self.funval1 = mfem.Vector(num_equation)
        self.funval2 = mfem.Vector(num_equation)        
        self.nor = mfem.Vector(dim)                
        self.fluxN = mfem.Vector(num_equation)                
        self.eip1 = mfem.IntegrationPoint()
        self.eip2 = mfem.IntegrationPoint()        
        super(FaceIntegrator, self).__init__()
        
    def AssembleFaceVector(self, el1, el2, Tr, elfun, elvect):
        num_equation = globals()['num_equation']        
        # Compute the term <F.n(u),[w]> on the interior faces.
        dof1 = el1.GetDof()
        dof2 = el2.GetDof()

        self.shape1.SetSize(dof1);
        self.shape2.SetSize(dof2);

        elvect.SetSize((dof1 + dof2) * num_equation);
        elvect.Assign(0.0)

        elfun1_mat = mfem.DenseMatrix(elfun.GetData(), dof1, num_equation)
        elfun2_mat = mfem.DenseMatrix(elfun[dof1*num_equation:].GetData(), dof2, num_equation)
        
        elvect1_mat = mfem.DenseMatrix(elvect.GetData(), dof1, num_equation)
        elvect2_mat = mfem.DenseMatrix(elvect[dof1*num_equation:].GetData(), dof2, num_equation)
        
        # Integration order calculation from DGTraceIntegrator
        if (Tr.Elem2No >= 0):
            intorder = (min(Tr.Elem1.OrderW(), Tr.Elem2.OrderW()) +
                       2*max(el1.GetOrder(), el2.GetOrder()))
        else:
            intorder = Tr.Elem1.OrderW() + 2*el1.GetOrder();

        if (el1.Space() == mfem.FunctionSpace().Pk):
            intorder += 1

        ir = mfem.IntRules.Get(Tr.GetGeometryType(), int(intorder))
        for i in range(ir.GetNPoints()):
            ip = ir.IntPoint(i)
            Tr.Loc1.Transform(ip, self.eip1)
            Tr.Loc2.Transform(ip, self.eip2)            

            # Calculate basis functions on both elements at the face
            el1.CalcShape(self.eip1, self.shape1)
            el2.CalcShape(self.eip2, self.shape2)

            # Interpolate elfun at the point
            elfun1_mat.MultTranspose(self.shape1, self.funval1)
            elfun2_mat.MultTranspose(self.shape2, self.funval2)
            Tr.Face.SetIntPoint(ip)

            # Get the normal vector and the flux on the face

            mfem.CalcOrtho(Tr.Face.Jacobian(), self.nor)

            mcs = self.rsolver.Eval(self.funval1, self.funval2, self.nor, self.fluxN)

            # Update max char speed
            if mcs > globals()['max_char_speed']:
                globals()['max_char_speed'] = mcs
                
            self.fluxN *= ip.weight;

            #
            fluxN = np.atleast_2d(self.fluxN.GetDataArray())
            shape1 = np.atleast_2d(self.shape1.GetDataArray())
            shape2 = np.atleast_2d(self.shape2.GetDataArray())                        
            mat1 = elvect1_mat.GetDataArray()
            mat2 = elvect2_mat.GetDataArray()
            mat1 -=  shape1.transpose().dot(fluxN)
            mat2 +=  shape2.transpose().dot(fluxN)            
            '''
            for k in range(num_equation):
                for s in range(dof1):
                    elvect1_mat[s, k] -= self.fluxN[k] * self.shape1[s]
                for s in range(dof2):                   
                    elvect2_mat[s, k] += self.fluxN[k] * self.shape2[s]
            '''
class RiemannSolver(object):
    def __init__(self):
        num_equation = globals()['num_equation']                
        self.flux1 = mfem.Vector(num_equation)
        self.flux2 = mfem.Vector(num_equation)
        
    def Eval(self, state1, state2, nor, flux):

        # NOTE: nor in general is not a unit normal
        dim = nor.Size()

        assert StateIsPhysical(state1, dim), ""
        assert StateIsPhysical(state2, dim), ""        

        maxE1 = ComputeMaxCharSpeed(state1, dim)
        maxE2 = ComputeMaxCharSpeed(state2, dim)
        maxE = max(maxE1, maxE2);

        ComputeFluxDotN(state1, nor, self.flux1)
        ComputeFluxDotN(state2, nor, self.flux2)

        normag = np.sqrt(np.sum(nor.GetDataArray()**2))

        '''
        for i in range(num_equation):
            flux[i] = (0.5 * (self.flux1[i] + self.flux2[i])
                       - 0.5 * maxE * (state2[i] - state1[i]) * normag)
        '''
        f = (0.5 * (self.flux1.GetDataArray() + self.flux2.GetDataArray())
             -  0.5 * maxE * (state2.GetDataArray() - state1.GetDataArray()) * normag)
        flux.Assign(f)

        return maxE;


def StateIsPhysical(state, dim):
    specific_heat_ratio = globals()["specific_heat_ratio"]
    
    den = state[0];
    den_vel = state[1:1+dim].GetDataArray()
    den_energy = state[1 + dim]

    if (den < 0):
        print("Negative density: "+ str(state.GetDataArray()))
        return False
    if (den_energy <= 0):
        print("Negative energy: "+ str(state.GetDataArray()))    
        return False

    den_vel2 = np.sum(den_vel**2)/den
    pres = (specific_heat_ratio - 1.0) * (den_energy - 0.5 * den_vel2);
    if (pres <= 0):
        print("Negative pressure: "+ str(state.GetDataArray()))            
        return False
    return True;

class InitialCondition(mfem.VectorPyCoefficient):
   def __init__(self, dim):
       mfem.VectorPyCoefficient.__init__(self, dim)
       
   def EvalValue(self, x):
       dim = x.shape[0]
       assert dim == 2, ""
       problem = globals()['problem']
       if (problem == 1):
           # "Fast vortex"
           radius = 0.2;
           Minf = 0.5;
           beta = 1. / 5.;
       elif (problem == 2):
           # "Slow vortex"
           radius = 0.2;
           Minf = 0.05;
           beta = 1. / 50.;
       else:
           assert False, "Cannot recognize problem. Options are: 1 - fast vortex, 2 - slow vortex"

       xc = 0.0;  yc = 0.0
       # Nice units
       vel_inf = 1.
       den_inf = 1.

       specific_heat_ratio = globals()["specific_heat_ratio"]
       gas_constant = globals()["gas_constant"]       

       pres_inf = (den_inf / specific_heat_ratio) * (vel_inf / Minf) * (vel_inf / Minf)
       temp_inf = pres_inf / (den_inf * gas_constant)

       r2rad = 0.0;
       r2rad += (x[0] - xc) * (x[0] - xc);
       r2rad += (x[1] - yc) * (x[1] - yc);
       r2rad /= (radius * radius);

       shrinv1 = 1.0 / (specific_heat_ratio - 1.);

       velX = vel_inf * (1 - beta * (x[1] - yc) / radius * np.exp(-0.5 * r2rad))
       velY = vel_inf * beta * (x[0] - xc) / radius * np.exp(-0.5 * r2rad);
       vel2 = velX * velX + velY * velY;
       
       specific_heat = gas_constant * specific_heat_ratio * shrinv1
       
       temp = temp_inf - (0.5 * (vel_inf * beta) *
                          (vel_inf * beta) / specific_heat * np.exp(-r2rad))

       den = den_inf * (temp/temp_inf)**shrinv1
       pres = den * gas_constant * temp;
       energy = shrinv1 * pres / den + 0.5 * vel2;

       y = np.array([den, den * velX, den * velY, den * energy])
       return y

def ComputePressure(state, dim):
    den = state[0];
    den_vel = state[1:1+dim].GetDataArray()
    den_energy = state[1 + dim]
    
    specific_heat_ratio = globals()["specific_heat_ratio"]
    den_vel2 = np.sum(den_vel**2)/den
    pres = (specific_heat_ratio - 1.0) * (den_energy - 0.5 * den_vel2)

    return pres

def ComputeFlux(state, dim, flux):
    from mpi4py import MPI
    num_procs = MPI.COMM_WORLD.size
    myid      = MPI.COMM_WORLD.rank

    den = state[0];
    den_vel = state[1:1+dim].GetDataArray()
    den_energy = state[1 + dim]

    assert StateIsPhysical(state, dim), ""

    pres = ComputePressure(state, dim)

    den_vel2 = np.atleast_2d(den_vel)
    fluxA = flux.GetDataArray()
    fluxA[0,:] = den_vel
    fluxA[1:1+dim, :] = den_vel2.transpose().dot(den_vel2) / den
    for d in range(dim): fluxA[1+d,d] += pres
    
    '''
    for d in range(dim):
       flux[0, d] = den_vel[d]
       for i in range(dim):       
          flux[1+i, d] = den_vel[i] * den_vel[d] / den;
       flux[1+d, d] += pres;
    '''
    
    H = (den_energy + pres) / den
    flux.GetDataArray()[1+dim, :] = den_vel * H

def ComputeFluxDotN(state, nor, fluxN):
    # NOTE: nor in general is not a unit normal
    dim = nor.Size()
    nor = nor.GetDataArray()
    fluxN = fluxN.GetDataArray()
    
    den = state[0]
    den_vel = state[1:1+dim].GetDataArray()
    den_energy = state[1 + dim]

   
    assert StateIsPhysical(state, dim), ""

    pres = ComputePressure(state, dim)    

    den_velN = np.sum(den_vel * nor)
    
    fluxN[0] = den_velN;
    fluxN[1:1+dim] = den_velN * den_vel / den + pres * nor

    H = (den_energy + pres) / den
    fluxN[1+dim] = den_velN * H

def ComputeMaxCharSpeed(state, dim):
    specific_heat_ratio = globals()["specific_heat_ratio"]
    
    den = state[0]    
    den_vel = state[1:1+dim].GetDataArray()
    den_energy = state[1 + dim]

    den_vel2 = np.sum(den_vel**2)
    den_vel2 /= den

    pres = ComputePressure(state, dim);
    sound = np.sqrt(specific_heat_ratio * pres / den);
    vel = np.sqrt(den_vel2 / den);

    return vel + sound;
    



# This file was automatically generated by SWIG (http://www.swig.org).
# Version 4.0.2
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.

from sys import version_info as _swig_python_version_info
if _swig_python_version_info < (2, 7, 0):
    raise RuntimeError("Python 2.7 or later required")

# Import the low-level C/C++ module
if __package__ or "." in __name__:
    from . import _blockoperator
else:
    import _blockoperator

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

_swig_new_instance_method = _blockoperator.SWIG_PyInstanceMethod_New
_swig_new_static_method = _blockoperator.SWIG_PyStaticMethod_New

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "thisown":
            self.this.own(value)
        elif name == "this":
            set(self, name, value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)


import weakref

import mfem._par.array
import mfem._par.mem_manager
import mfem._par.vector
import mfem._par.operators

def Opr2BlockOpr(op):
    r"""Opr2BlockOpr(Operator op) -> BlockOperator"""
    return _blockoperator.Opr2BlockOpr(op)
Opr2BlockOpr = _blockoperator.Opr2BlockOpr

def Opr2SparseMat(op):
    r"""Opr2SparseMat(Operator op) -> mfem::SparseMatrix *"""
    return _blockoperator.Opr2SparseMat(op)
Opr2SparseMat = _blockoperator.Opr2SparseMat

def Opr2HypreParMat(op):
    r"""Opr2HypreParMat(Operator op) -> mfem::HypreParMatrix *"""
    return _blockoperator.Opr2HypreParMat(op)
Opr2HypreParMat = _blockoperator.Opr2HypreParMat
class BlockOperator(mfem._par.operators.Operator):
    r"""Proxy of C++ mfem::BlockOperator class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(BlockOperator self, intArray offsets) -> BlockOperator
        __init__(BlockOperator self, intArray row_offsets, intArray col_offsets) -> BlockOperator
        """
        _blockoperator.BlockOperator_swiginit(self, _blockoperator.new_BlockOperator(*args))

        from mfem.par import intArray  
        if len(args) == 1:
           if isinstance(args[0], intArray):
               self._offsets = args[0]
        if len(args) == 2:
           if (isinstance(args[0], intArray) and
               isinstance(args[1], intArray)):
               self._offsets = (args[0], args[1])




    def SetDiagonalBlock(self, iblock, op, c=1.0):
        r"""SetDiagonalBlock(BlockOperator self, int iblock, Operator op, double c=1.0)"""
        val = _blockoperator.BlockOperator_SetDiagonalBlock(self, iblock, op, c)

        if not hasattr(self, '_linked_op'):
           self._linked_op = {}
        self._linked_op[iblock, iblock] = op


        return val


    def SetBlock(self, iRow, iCol, op, c=1.0):
        r"""SetBlock(BlockOperator self, int iRow, int iCol, Operator op, double c=1.0)"""
        val = _blockoperator.BlockOperator_SetBlock(self, iRow, iCol, op, c)

        if not hasattr(self, '_linked_op'):
           self._linked_op = {}
        self._linked_op[iRow, iCol] = op


        return val


    def NumRowBlocks(self):
        r"""NumRowBlocks(BlockOperator self) -> int"""
        return _blockoperator.BlockOperator_NumRowBlocks(self)
    NumRowBlocks = _swig_new_instance_method(_blockoperator.BlockOperator_NumRowBlocks)

    def NumColBlocks(self):
        r"""NumColBlocks(BlockOperator self) -> int"""
        return _blockoperator.BlockOperator_NumColBlocks(self)
    NumColBlocks = _swig_new_instance_method(_blockoperator.BlockOperator_NumColBlocks)

    def IsZeroBlock(self, i, j):
        r"""IsZeroBlock(BlockOperator self, int i, int j) -> int"""
        return _blockoperator.BlockOperator_IsZeroBlock(self, i, j)
    IsZeroBlock = _swig_new_instance_method(_blockoperator.BlockOperator_IsZeroBlock)

    def GetBlock(self, *args):
        r"""
        GetBlock(BlockOperator self, int i, int j) -> Operator
        GetBlock(BlockOperator self, int i, int j) -> Operator
        """
        return _blockoperator.BlockOperator_GetBlock(self, *args)
    GetBlock = _swig_new_instance_method(_blockoperator.BlockOperator_GetBlock)

    def GetBlockCoef(self, i, j):
        r"""GetBlockCoef(BlockOperator self, int i, int j) -> double"""
        return _blockoperator.BlockOperator_GetBlockCoef(self, i, j)
    GetBlockCoef = _swig_new_instance_method(_blockoperator.BlockOperator_GetBlockCoef)

    def SetBlockCoef(self, i, j, c):
        r"""SetBlockCoef(BlockOperator self, int i, int j, double c)"""
        return _blockoperator.BlockOperator_SetBlockCoef(self, i, j, c)
    SetBlockCoef = _swig_new_instance_method(_blockoperator.BlockOperator_SetBlockCoef)

    def RowOffsets(self, *args):
        r"""
        RowOffsets(BlockOperator self) -> intArray
        RowOffsets(BlockOperator self) -> intArray
        """
        return _blockoperator.BlockOperator_RowOffsets(self, *args)
    RowOffsets = _swig_new_instance_method(_blockoperator.BlockOperator_RowOffsets)

    def ColOffsets(self, *args):
        r"""
        ColOffsets(BlockOperator self) -> intArray
        ColOffsets(BlockOperator self) -> intArray
        """
        return _blockoperator.BlockOperator_ColOffsets(self, *args)
    ColOffsets = _swig_new_instance_method(_blockoperator.BlockOperator_ColOffsets)

    def Mult(self, x, y):
        r"""Mult(BlockOperator self, Vector x, Vector y)"""
        return _blockoperator.BlockOperator_Mult(self, x, y)
    Mult = _swig_new_instance_method(_blockoperator.BlockOperator_Mult)

    def MultTranspose(self, x, y):
        r"""MultTranspose(BlockOperator self, Vector x, Vector y)"""
        return _blockoperator.BlockOperator_MultTranspose(self, x, y)
    MultTranspose = _swig_new_instance_method(_blockoperator.BlockOperator_MultTranspose)
    __swig_destroy__ = _blockoperator.delete_BlockOperator
    owns_blocks = property(_blockoperator.BlockOperator_owns_blocks_get, _blockoperator.BlockOperator_owns_blocks_set, doc=r"""owns_blocks : int""")

# Register BlockOperator in _blockoperator:
_blockoperator.BlockOperator_swigregister(BlockOperator)

class BlockDiagonalPreconditioner(mfem._par.operators.Solver):
    r"""Proxy of C++ mfem::BlockDiagonalPreconditioner class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, offsets):
        r"""__init__(BlockDiagonalPreconditioner self, intArray offsets) -> BlockDiagonalPreconditioner"""
        _blockoperator.BlockDiagonalPreconditioner_swiginit(self, _blockoperator.new_BlockDiagonalPreconditioner(offsets))

        self._offsets = offsets




    def SetDiagonalBlock(self, iblock, op):
        r"""SetDiagonalBlock(BlockDiagonalPreconditioner self, int iblock, Operator op)"""
        val = _blockoperator.BlockDiagonalPreconditioner_SetDiagonalBlock(self, iblock, op)

        if not hasattr(self, '_linked_op'):
           self._linked_op = {}
        self._linked_op[iblock, iblock] = op


        return val


    def SetOperator(self, op):
        r"""SetOperator(BlockDiagonalPreconditioner self, Operator op)"""
        return _blockoperator.BlockDiagonalPreconditioner_SetOperator(self, op)
    SetOperator = _swig_new_instance_method(_blockoperator.BlockDiagonalPreconditioner_SetOperator)

    def NumBlocks(self):
        r"""NumBlocks(BlockDiagonalPreconditioner self) -> int"""
        return _blockoperator.BlockDiagonalPreconditioner_NumBlocks(self)
    NumBlocks = _swig_new_instance_method(_blockoperator.BlockDiagonalPreconditioner_NumBlocks)

    def GetDiagonalBlock(self, *args):
        r"""
        GetDiagonalBlock(BlockDiagonalPreconditioner self, int iblock) -> Operator
        GetDiagonalBlock(BlockDiagonalPreconditioner self, int iblock) -> Operator
        """
        return _blockoperator.BlockDiagonalPreconditioner_GetDiagonalBlock(self, *args)
    GetDiagonalBlock = _swig_new_instance_method(_blockoperator.BlockDiagonalPreconditioner_GetDiagonalBlock)

    def Offsets(self, *args):
        r"""
        Offsets(BlockDiagonalPreconditioner self) -> intArray
        Offsets(BlockDiagonalPreconditioner self) -> intArray
        """
        return _blockoperator.BlockDiagonalPreconditioner_Offsets(self, *args)
    Offsets = _swig_new_instance_method(_blockoperator.BlockDiagonalPreconditioner_Offsets)

    def Mult(self, x, y):
        r"""Mult(BlockDiagonalPreconditioner self, Vector x, Vector y)"""
        return _blockoperator.BlockDiagonalPreconditioner_Mult(self, x, y)
    Mult = _swig_new_instance_method(_blockoperator.BlockDiagonalPreconditioner_Mult)

    def MultTranspose(self, x, y):
        r"""MultTranspose(BlockDiagonalPreconditioner self, Vector x, Vector y)"""
        return _blockoperator.BlockDiagonalPreconditioner_MultTranspose(self, x, y)
    MultTranspose = _swig_new_instance_method(_blockoperator.BlockDiagonalPreconditioner_MultTranspose)
    __swig_destroy__ = _blockoperator.delete_BlockDiagonalPreconditioner
    owns_blocks = property(_blockoperator.BlockDiagonalPreconditioner_owns_blocks_get, _blockoperator.BlockDiagonalPreconditioner_owns_blocks_set, doc=r"""owns_blocks : int""")

# Register BlockDiagonalPreconditioner in _blockoperator:
_blockoperator.BlockDiagonalPreconditioner_swigregister(BlockDiagonalPreconditioner)

class BlockLowerTriangularPreconditioner(mfem._par.operators.Solver):
    r"""Proxy of C++ mfem::BlockLowerTriangularPreconditioner class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, offsets):
        r"""__init__(BlockLowerTriangularPreconditioner self, intArray offsets) -> BlockLowerTriangularPreconditioner"""
        _blockoperator.BlockLowerTriangularPreconditioner_swiginit(self, _blockoperator.new_BlockLowerTriangularPreconditioner(offsets))

        self._offsets = offsets




    def SetDiagonalBlock(self, iblock, op):
        r"""SetDiagonalBlock(BlockLowerTriangularPreconditioner self, int iblock, Operator op)"""
        val = _blockoperator.BlockLowerTriangularPreconditioner_SetDiagonalBlock(self, iblock, op)

        if not hasattr(self, '_linked_op'):
           self._linked_op = {}
        self._linked_op[iblock, iblock] = op


        return val


    def SetBlock(self, iRow, iCol, op):
        r"""SetBlock(BlockLowerTriangularPreconditioner self, int iRow, int iCol, Operator op)"""

        if not (iRow > iCol):
            raise ValueError("can not set upper triangle")


        val = _blockoperator.BlockLowerTriangularPreconditioner_SetBlock(self, iRow, iCol, op)

        if not hasattr(self, '_linked_op'):
           self._linked_op = {}

        self._linked_op[iRow, iCol] = op


        return val


    def SetOperator(self, op):
        r"""SetOperator(BlockLowerTriangularPreconditioner self, Operator op)"""
        return _blockoperator.BlockLowerTriangularPreconditioner_SetOperator(self, op)
    SetOperator = _swig_new_instance_method(_blockoperator.BlockLowerTriangularPreconditioner_SetOperator)

    def NumBlocks(self):
        r"""NumBlocks(BlockLowerTriangularPreconditioner self) -> int"""
        return _blockoperator.BlockLowerTriangularPreconditioner_NumBlocks(self)
    NumBlocks = _swig_new_instance_method(_blockoperator.BlockLowerTriangularPreconditioner_NumBlocks)

    def GetBlock(self, iblock, jblock):
        r"""GetBlock(BlockLowerTriangularPreconditioner self, int iblock, int jblock) -> Operator"""
        return _blockoperator.BlockLowerTriangularPreconditioner_GetBlock(self, iblock, jblock)
    GetBlock = _swig_new_instance_method(_blockoperator.BlockLowerTriangularPreconditioner_GetBlock)

    def Offsets(self):
        r"""Offsets(BlockLowerTriangularPreconditioner self) -> intArray"""
        return _blockoperator.BlockLowerTriangularPreconditioner_Offsets(self)
    Offsets = _swig_new_instance_method(_blockoperator.BlockLowerTriangularPreconditioner_Offsets)

    def Mult(self, x, y):
        r"""Mult(BlockLowerTriangularPreconditioner self, Vector x, Vector y)"""
        return _blockoperator.BlockLowerTriangularPreconditioner_Mult(self, x, y)
    Mult = _swig_new_instance_method(_blockoperator.BlockLowerTriangularPreconditioner_Mult)

    def MultTranspose(self, x, y):
        r"""MultTranspose(BlockLowerTriangularPreconditioner self, Vector x, Vector y)"""
        return _blockoperator.BlockLowerTriangularPreconditioner_MultTranspose(self, x, y)
    MultTranspose = _swig_new_instance_method(_blockoperator.BlockLowerTriangularPreconditioner_MultTranspose)
    __swig_destroy__ = _blockoperator.delete_BlockLowerTriangularPreconditioner
    owns_blocks = property(_blockoperator.BlockLowerTriangularPreconditioner_owns_blocks_get, _blockoperator.BlockLowerTriangularPreconditioner_owns_blocks_set, doc=r"""owns_blocks : int""")

# Register BlockLowerTriangularPreconditioner in _blockoperator:
_blockoperator.BlockLowerTriangularPreconditioner_swigregister(BlockLowerTriangularPreconditioner)






# This file was automatically generated by SWIG (http://www.swig.org).
# Version 4.0.2
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.

from sys import version_info as _swig_python_version_info
if _swig_python_version_info < (2, 7, 0):
    raise RuntimeError("Python 2.7 or later required")

# Import the low-level C/C++ module
if __package__ or "." in __name__:
    from . import _linearform
else:
    import _linearform

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

_swig_new_instance_method = _linearform.SWIG_PyInstanceMethod_New
_swig_new_static_method = _linearform.SWIG_PyStaticMethod_New

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "thisown":
            self.this.own(value)
        elif name == "this":
            set(self, name, value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)


import weakref

class intp(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self):
        _linearform.intp_swiginit(self, _linearform.new_intp())
    __swig_destroy__ = _linearform.delete_intp

    def assign(self, value):
        return _linearform.intp_assign(self, value)
    assign = _swig_new_instance_method(_linearform.intp_assign)

    def value(self):
        return _linearform.intp_value(self)
    value = _swig_new_instance_method(_linearform.intp_value)

    def cast(self):
        return _linearform.intp_cast(self)
    cast = _swig_new_instance_method(_linearform.intp_cast)

    @staticmethod
    def frompointer(t):
        return _linearform.intp_frompointer(t)
    frompointer = _swig_new_static_method(_linearform.intp_frompointer)

# Register intp in _linearform:
_linearform.intp_swigregister(intp)

def intp_frompointer(t):
    return _linearform.intp_frompointer(t)
intp_frompointer = _linearform.intp_frompointer

class doublep(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self):
        _linearform.doublep_swiginit(self, _linearform.new_doublep())
    __swig_destroy__ = _linearform.delete_doublep

    def assign(self, value):
        return _linearform.doublep_assign(self, value)
    assign = _swig_new_instance_method(_linearform.doublep_assign)

    def value(self):
        return _linearform.doublep_value(self)
    value = _swig_new_instance_method(_linearform.doublep_value)

    def cast(self):
        return _linearform.doublep_cast(self)
    cast = _swig_new_instance_method(_linearform.doublep_cast)

    @staticmethod
    def frompointer(t):
        return _linearform.doublep_frompointer(t)
    frompointer = _swig_new_static_method(_linearform.doublep_frompointer)

# Register doublep in _linearform:
_linearform.doublep_swigregister(doublep)

def doublep_frompointer(t):
    return _linearform.doublep_frompointer(t)
doublep_frompointer = _linearform.doublep_frompointer

import mfem._ser.coefficient
import mfem._ser.globals
import mfem._ser.array
import mfem._ser.mem_manager
import mfem._ser.matrix
import mfem._ser.vector
import mfem._ser.operators
import mfem._ser.intrules
import mfem._ser.sparsemat
import mfem._ser.densemat
import mfem._ser.eltrans
import mfem._ser.fe
import mfem._ser.geom
import mfem._ser.mesh
import mfem._ser.sort_pairs
import mfem._ser.ncmesh
import mfem._ser.gridfunc
import mfem._ser.fespace
import mfem._ser.fe_coll
import mfem._ser.lininteg
import mfem._ser.handle
import mfem._ser.restriction
import mfem._ser.element
import mfem._ser.table
import mfem._ser.hash
import mfem._ser.bilininteg
import mfem._ser.nonlininteg
import mfem._ser.vertex
import mfem._ser.vtk
class LinearForm(mfem._ser.vector.Vector):
    r"""Proxy of C++ mfem::LinearForm class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(LinearForm self, FiniteElementSpace f) -> LinearForm
        __init__(LinearForm self, FiniteElementSpace f, LinearForm lf) -> LinearForm
        __init__(LinearForm self) -> LinearForm
        __init__(LinearForm self, FiniteElementSpace f, double * data) -> LinearForm
        """
        _linearform.LinearForm_swiginit(self, _linearform.new_LinearForm(*args))

    def GetFES(self):
        r"""GetFES(LinearForm self) -> FiniteElementSpace"""

        import warnings
        warnings.warn("mfem::LinearForm::GetFES() is deprecated",
                      DeprecationWarning,)


        return _linearform.LinearForm_GetFES(self)


    def FESpace(self, *args):
        r"""
        FESpace(LinearForm self) -> FiniteElementSpace
        FESpace(LinearForm self) -> FiniteElementSpace
        """
        return _linearform.LinearForm_FESpace(self, *args)
    FESpace = _swig_new_instance_method(_linearform.LinearForm_FESpace)

    def AddDomainIntegrator(self, lfi):
        r"""AddDomainIntegrator(LinearForm self, LinearFormIntegrator lfi)"""

        if not hasattr(self, "_integrators"): self._integrators = []
        self._integrators.append(lfi)
        lfi.thisown=0 


        return _linearform.LinearForm_AddDomainIntegrator(self, lfi)


    def AddBoundaryIntegrator(self, *args):
        r"""
        AddBoundaryIntegrator(LinearForm self, LinearFormIntegrator lfi)
        AddBoundaryIntegrator(LinearForm self, LinearFormIntegrator lfi, intArray bdr_attr_marker)
        """

        if not hasattr(self, "_integrators"): self._integrators = []
        lfi = args[0]	     
        self._integrators.append(lfi)
        lfi.thisown=0


        return _linearform.LinearForm_AddBoundaryIntegrator(self, *args)


    def AddBdrFaceIntegrator(self, *args):
        r"""
        AddBdrFaceIntegrator(LinearForm self, LinearFormIntegrator lfi)
        AddBdrFaceIntegrator(LinearForm self, LinearFormIntegrator lfi, intArray bdr_attr_marker)
        """

        if not hasattr(self, "_integrators"): self._integrators = []
        lfi = args[0]
        self._integrators.append(lfi)
        lfi.thisown=0 


        return _linearform.LinearForm_AddBdrFaceIntegrator(self, *args)


    def GetDLFI(self):
        r"""GetDLFI(LinearForm self) -> mfem::Array< mfem::LinearFormIntegrator * > *"""
        return _linearform.LinearForm_GetDLFI(self)
    GetDLFI = _swig_new_instance_method(_linearform.LinearForm_GetDLFI)

    def GetDLFI_Delta(self):
        r"""GetDLFI_Delta(LinearForm self) -> mfem::Array< mfem::DeltaLFIntegrator * > *"""
        return _linearform.LinearForm_GetDLFI_Delta(self)
    GetDLFI_Delta = _swig_new_instance_method(_linearform.LinearForm_GetDLFI_Delta)

    def GetBLFI(self):
        r"""GetBLFI(LinearForm self) -> mfem::Array< mfem::LinearFormIntegrator * > *"""
        return _linearform.LinearForm_GetBLFI(self)
    GetBLFI = _swig_new_instance_method(_linearform.LinearForm_GetBLFI)

    def GetFLFI(self):
        r"""GetFLFI(LinearForm self) -> mfem::Array< mfem::LinearFormIntegrator * > *"""
        return _linearform.LinearForm_GetFLFI(self)
    GetFLFI = _swig_new_instance_method(_linearform.LinearForm_GetFLFI)

    def GetFLFI_Marker(self):
        r"""GetFLFI_Marker(LinearForm self) -> mfem::Array< mfem::Array< int > * > *"""
        return _linearform.LinearForm_GetFLFI_Marker(self)
    GetFLFI_Marker = _swig_new_instance_method(_linearform.LinearForm_GetFLFI_Marker)

    def Assemble(self):
        r"""Assemble(LinearForm self)"""
        return _linearform.LinearForm_Assemble(self)
    Assemble = _swig_new_instance_method(_linearform.LinearForm_Assemble)

    def AssembleDelta(self):
        r"""AssembleDelta(LinearForm self)"""
        return _linearform.LinearForm_AssembleDelta(self)
    AssembleDelta = _swig_new_instance_method(_linearform.LinearForm_AssembleDelta)

    def Update(self, *args):
        r"""
        Update(LinearForm self)
        Update(LinearForm self, FiniteElementSpace f)
        Update(LinearForm self, FiniteElementSpace f, Vector v, int v_offset)
        """
        return _linearform.LinearForm_Update(self, *args)
    Update = _swig_new_instance_method(_linearform.LinearForm_Update)

    def MakeRef(self, f, v, v_offset):
        r"""MakeRef(LinearForm self, FiniteElementSpace f, Vector v, int v_offset)"""
        return _linearform.LinearForm_MakeRef(self, f, v, v_offset)
    MakeRef = _swig_new_instance_method(_linearform.LinearForm_MakeRef)

    def __call__(self, gf):
        r"""__call__(LinearForm self, GridFunction gf) -> double"""
        return _linearform.LinearForm___call__(self, gf)
    __call__ = _swig_new_instance_method(_linearform.LinearForm___call__)
    __swig_destroy__ = _linearform.delete_LinearForm

# Register LinearForm in _linearform:
_linearform.LinearForm_swigregister(LinearForm)






# This file was automatically generated by SWIG (http://www.swig.org).
# Version 4.0.2
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.

from sys import version_info as _swig_python_version_info
if _swig_python_version_info < (2, 7, 0):
    raise RuntimeError("Python 2.7 or later required")

# Import the low-level C/C++ module
if __package__ or "." in __name__:
    from . import _table
else:
    import _table

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

_swig_new_instance_method = _table.SWIG_PyInstanceMethod_New
_swig_new_static_method = _table.SWIG_PyStaticMethod_New

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "thisown":
            self.this.own(value)
        elif name == "this":
            set(self, name, value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)


import mfem._ser.array
import mfem._ser.mem_manager
class Connection(object):
    r"""Proxy of C++ mfem::Connection class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    _from = property(_table.Connection__from_get, _table.Connection__from_set, doc=r"""_from : int""")
    to = property(_table.Connection_to_get, _table.Connection_to_set, doc=r"""to : int""")

    def __init__(self, *args):
        r"""
        __init__(Connection self) -> Connection
        __init__(Connection self, int _from, int to) -> Connection
        """
        _table.Connection_swiginit(self, _table.new_Connection(*args))

    def __eq__(self, rhs):
        r"""__eq__(Connection self, Connection rhs) -> bool"""
        return _table.Connection___eq__(self, rhs)
    __eq__ = _swig_new_instance_method(_table.Connection___eq__)

    def __lt__(self, rhs):
        r"""__lt__(Connection self, Connection rhs) -> bool"""
        return _table.Connection___lt__(self, rhs)
    __lt__ = _swig_new_instance_method(_table.Connection___lt__)
    __swig_destroy__ = _table.delete_Connection

# Register Connection in _table:
_table.Connection_swigregister(Connection)

class Table(object):
    r"""Proxy of C++ mfem::Table class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(Table self) -> Table
        __init__(Table self, Table arg2) -> Table
        __init__(Table self, int dim, int connections_per_row=3) -> Table
        __init__(Table self, int nrows, mfem::Array< mfem::Connection > & list) -> Table
        __init__(Table self, int nrows, int * partitioning) -> Table
        """
        _table.Table_swiginit(self, _table.new_Table(*args))

    def MakeI(self, nrows):
        r"""MakeI(Table self, int nrows)"""
        return _table.Table_MakeI(self, nrows)
    MakeI = _swig_new_instance_method(_table.Table_MakeI)

    def AddAColumnInRow(self, r):
        r"""AddAColumnInRow(Table self, int r)"""
        return _table.Table_AddAColumnInRow(self, r)
    AddAColumnInRow = _swig_new_instance_method(_table.Table_AddAColumnInRow)

    def AddColumnsInRow(self, r, ncol):
        r"""AddColumnsInRow(Table self, int r, int ncol)"""
        return _table.Table_AddColumnsInRow(self, r, ncol)
    AddColumnsInRow = _swig_new_instance_method(_table.Table_AddColumnsInRow)

    def MakeJ(self):
        r"""MakeJ(Table self)"""
        return _table.Table_MakeJ(self)
    MakeJ = _swig_new_instance_method(_table.Table_MakeJ)

    def AddConnection(self, r, c):
        r"""AddConnection(Table self, int r, int c)"""
        return _table.Table_AddConnection(self, r, c)
    AddConnection = _swig_new_instance_method(_table.Table_AddConnection)

    def AddConnections(self, r, c, nc):
        r"""AddConnections(Table self, int r, int const * c, int nc)"""
        return _table.Table_AddConnections(self, r, c, nc)
    AddConnections = _swig_new_instance_method(_table.Table_AddConnections)

    def ShiftUpI(self):
        r"""ShiftUpI(Table self)"""
        return _table.Table_ShiftUpI(self)
    ShiftUpI = _swig_new_instance_method(_table.Table_ShiftUpI)

    def SetSize(self, dim, connections_per_row):
        r"""SetSize(Table self, int dim, int connections_per_row)"""
        return _table.Table_SetSize(self, dim, connections_per_row)
    SetSize = _swig_new_instance_method(_table.Table_SetSize)

    def SetDims(self, rows, nnz):
        r"""SetDims(Table self, int rows, int nnz)"""
        return _table.Table_SetDims(self, rows, nnz)
    SetDims = _swig_new_instance_method(_table.Table_SetDims)

    def Size(self):
        r"""Size(Table self) -> int"""
        return _table.Table_Size(self)
    Size = _swig_new_instance_method(_table.Table_Size)

    def Size_of_connections(self):
        r"""Size_of_connections(Table self) -> int"""
        return _table.Table_Size_of_connections(self)
    Size_of_connections = _swig_new_instance_method(_table.Table_Size_of_connections)

    def __call__(self, i, j):
        r"""__call__(Table self, int i, int j) -> int"""
        return _table.Table___call__(self, i, j)
    __call__ = _swig_new_instance_method(_table.Table___call__)

    def RowSize(self, i):
        r"""RowSize(Table self, int i) -> int"""
        return _table.Table_RowSize(self, i)
    RowSize = _swig_new_instance_method(_table.Table_RowSize)

    def GetRow(self, *args):
        r"""
        GetRow(Table self, int i, intArray row)
        GetRow(Table self, int i) -> int const
        GetRow(Table self, int i) -> int *
        """
        return _table.Table_GetRow(self, *args)
    GetRow = _swig_new_instance_method(_table.Table_GetRow)

    def GetI(self, *args):
        r"""
        GetI(Table self) -> int
        GetI(Table self) -> int const *
        """
        return _table.Table_GetI(self, *args)
    GetI = _swig_new_instance_method(_table.Table_GetI)

    def GetJ(self, *args):
        r"""
        GetJ(Table self) -> int
        GetJ(Table self) -> int const *
        """
        return _table.Table_GetJ(self, *args)
    GetJ = _swig_new_instance_method(_table.Table_GetJ)

    def GetIMemory(self, *args):
        r"""
        GetIMemory(Table self) -> mfem::Memory< int >
        GetIMemory(Table self) -> mfem::Memory< int > const &
        """
        return _table.Table_GetIMemory(self, *args)
    GetIMemory = _swig_new_instance_method(_table.Table_GetIMemory)

    def GetJMemory(self, *args):
        r"""
        GetJMemory(Table self) -> mfem::Memory< int >
        GetJMemory(Table self) -> mfem::Memory< int > const &
        """
        return _table.Table_GetJMemory(self, *args)
    GetJMemory = _swig_new_instance_method(_table.Table_GetJMemory)

    def SortRows(self):
        r"""SortRows(Table self)"""
        return _table.Table_SortRows(self)
    SortRows = _swig_new_instance_method(_table.Table_SortRows)

    def SetIJ(self, newI, newJ, newsize=-1):
        r"""SetIJ(Table self, int * newI, int * newJ, int newsize=-1)"""
        return _table.Table_SetIJ(self, newI, newJ, newsize)
    SetIJ = _swig_new_instance_method(_table.Table_SetIJ)

    def Push(self, i, j):
        r"""Push(Table self, int i, int j) -> int"""
        return _table.Table_Push(self, i, j)
    Push = _swig_new_instance_method(_table.Table_Push)

    def Finalize(self):
        r"""Finalize(Table self)"""
        return _table.Table_Finalize(self)
    Finalize = _swig_new_instance_method(_table.Table_Finalize)

    def MakeFromList(self, nrows, list):
        r"""MakeFromList(Table self, int nrows, mfem::Array< mfem::Connection > const & list)"""
        return _table.Table_MakeFromList(self, nrows, list)
    MakeFromList = _swig_new_instance_method(_table.Table_MakeFromList)

    def Width(self):
        r"""Width(Table self) -> int"""
        return _table.Table_Width(self)
    Width = _swig_new_instance_method(_table.Table_Width)

    def LoseData(self):
        r"""LoseData(Table self)"""
        return _table.Table_LoseData(self)
    LoseData = _swig_new_instance_method(_table.Table_LoseData)

    def Load(self, _in):
        r"""Load(Table self, std::istream & _in)"""
        return _table.Table_Load(self, _in)
    Load = _swig_new_instance_method(_table.Table_Load)

    def Copy(self, copy):
        r"""Copy(Table self, Table copy)"""
        return _table.Table_Copy(self, copy)
    Copy = _swig_new_instance_method(_table.Table_Copy)

    def Swap(self, other):
        r"""Swap(Table self, Table other)"""
        return _table.Table_Swap(self, other)
    Swap = _swig_new_instance_method(_table.Table_Swap)

    def Clear(self):
        r"""Clear(Table self)"""
        return _table.Table_Clear(self)
    Clear = _swig_new_instance_method(_table.Table_Clear)

    def MemoryUsage(self):
        r"""MemoryUsage(Table self) -> long"""
        return _table.Table_MemoryUsage(self)
    MemoryUsage = _swig_new_instance_method(_table.Table_MemoryUsage)
    __swig_destroy__ = _table.delete_Table

    def GetRowList(self, i):
        r"""GetRowList(Table self, int i) -> PyObject *"""
        return _table.Table_GetRowList(self, i)
    GetRowList = _swig_new_instance_method(_table.Table_GetRowList)

    def Print(self, *args):
        r"""
        Print(Table self, std::ostream & out=mfem::out, int width=4)
        Print(Table self, char const * file, int precision=8)
        """
        return _table.Table_Print(self, *args)
    Print = _swig_new_instance_method(_table.Table_Print)

    def PrintMatlab(self, *args):
        r"""
        PrintMatlab(Table self, std::ostream & out)
        PrintMatlab(Table self, char const * file, int precision=8)
        """
        return _table.Table_PrintMatlab(self, *args)
    PrintMatlab = _swig_new_instance_method(_table.Table_PrintMatlab)

    def Save(self, *args):
        r"""
        Save(Table self, std::ostream & out)
        Save(Table self, char const * file, int precision=8)
        Save(Table self)
        """
        return _table.Table_Save(self, *args)
    Save = _swig_new_instance_method(_table.Table_Save)

# Register Table in _table:
_table.Table_swigregister(Table)

class STable(Table):
    r"""Proxy of C++ mfem::STable class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, dim, connections_per_row=3):
        r"""__init__(STable self, int dim, int connections_per_row=3) -> STable"""
        _table.STable_swiginit(self, _table.new_STable(dim, connections_per_row))

    def __call__(self, i, j):
        r"""__call__(STable self, int i, int j) -> int"""
        return _table.STable___call__(self, i, j)
    __call__ = _swig_new_instance_method(_table.STable___call__)

    def Push(self, i, j):
        r"""Push(STable self, int i, int j) -> int"""
        return _table.STable_Push(self, i, j)
    Push = _swig_new_instance_method(_table.STable_Push)
    __swig_destroy__ = _table.delete_STable

# Register STable in _table:
_table.STable_swigregister(STable)

class DSTable(object):
    r"""Proxy of C++ mfem::DSTable class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, nrows):
        r"""__init__(DSTable self, int nrows) -> DSTable"""
        _table.DSTable_swiginit(self, _table.new_DSTable(nrows))

    def NumberOfRows(self):
        r"""NumberOfRows(DSTable self) -> int"""
        return _table.DSTable_NumberOfRows(self)
    NumberOfRows = _swig_new_instance_method(_table.DSTable_NumberOfRows)

    def NumberOfEntries(self):
        r"""NumberOfEntries(DSTable self) -> int"""
        return _table.DSTable_NumberOfEntries(self)
    NumberOfEntries = _swig_new_instance_method(_table.DSTable_NumberOfEntries)

    def Push(self, a, b):
        r"""Push(DSTable self, int a, int b) -> int"""
        return _table.DSTable_Push(self, a, b)
    Push = _swig_new_instance_method(_table.DSTable_Push)

    def __call__(self, a, b):
        r"""__call__(DSTable self, int a, int b) -> int"""
        return _table.DSTable___call__(self, a, b)
    __call__ = _swig_new_instance_method(_table.DSTable___call__)
    __swig_destroy__ = _table.delete_DSTable

# Register DSTable in _table:
_table.DSTable_swigregister(DSTable)






# This file was automatically generated by SWIG (http://www.swig.org).
# Version 4.0.2
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.

from sys import version_info as _swig_python_version_info
if _swig_python_version_info < (2, 7, 0):
    raise RuntimeError("Python 2.7 or later required")

# Import the low-level C/C++ module
if __package__ or "." in __name__:
    from . import _triangle
else:
    import _triangle

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

_swig_new_instance_method = _triangle.SWIG_PyInstanceMethod_New
_swig_new_static_method = _triangle.SWIG_PyStaticMethod_New

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "thisown":
            self.this.own(value)
        elif name == "this":
            set(self, name, value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)


import weakref

import mfem._par.fe
import mfem._par.array
import mfem._par.mem_manager
import mfem._par.vector
import mfem._par.geom
import mfem._par.intrules
import mfem._par.densemat
import mfem._par.operators
import mfem._par.matrix
import mfem._par.sparsemat
import mfem._par.element
import mfem._par.globals
import mfem._par.table
import mfem._par.hash
class Triangle(mfem._par.element.Element):
    r"""Proxy of C++ mfem::Triangle class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(Triangle self) -> Triangle
        __init__(Triangle self, int const * ind, int attr=1) -> Triangle
        __init__(Triangle self, int ind1, int ind2, int ind3, int attr=1) -> Triangle
        """
        _triangle.Triangle_swiginit(self, _triangle.new_Triangle(*args))

    def GetType(self):
        r"""GetType(Triangle self) -> mfem::Element::Type"""
        return _triangle.Triangle_GetType(self)
    GetType = _swig_new_instance_method(_triangle.Triangle_GetType)

    def NeedRefinement(self, v_to_v):
        r"""NeedRefinement(Triangle self, mfem::HashTable< mfem::Hashed2 > & v_to_v) -> int"""
        return _triangle.Triangle_NeedRefinement(self, v_to_v)
    NeedRefinement = _swig_new_instance_method(_triangle.Triangle_NeedRefinement)

    def SetVertices(self, ind):
        r"""SetVertices(Triangle self, int const * ind)"""
        return _triangle.Triangle_SetVertices(self, ind)
    SetVertices = _swig_new_instance_method(_triangle.Triangle_SetVertices)

    def MarkEdge(self, *args):
        r"""
        MarkEdge(Triangle self, DenseMatrix pmat)
        MarkEdge(Triangle self, int * indices, DSTable v_to_v, int const * length)
        MarkEdge(Triangle self, DSTable v_to_v, int const * length)
        """
        return _triangle.Triangle_MarkEdge(self, *args)
    MarkEdge = _swig_new_instance_method(_triangle.Triangle_MarkEdge)

    def ResetTransform(self, tr):
        r"""ResetTransform(Triangle self, int tr)"""
        return _triangle.Triangle_ResetTransform(self, tr)
    ResetTransform = _swig_new_instance_method(_triangle.Triangle_ResetTransform)

    def GetTransform(self):
        r"""GetTransform(Triangle self) -> unsigned int"""
        return _triangle.Triangle_GetTransform(self)
    GetTransform = _swig_new_instance_method(_triangle.Triangle_GetTransform)

    def PushTransform(self, tr):
        r"""PushTransform(Triangle self, int tr)"""
        return _triangle.Triangle_PushTransform(self, tr)
    PushTransform = _swig_new_instance_method(_triangle.Triangle_PushTransform)

    @staticmethod
    def GetPointMatrix(transform, pm):
        r"""GetPointMatrix(unsigned int transform, DenseMatrix pm)"""
        return _triangle.Triangle_GetPointMatrix(transform, pm)
    GetPointMatrix = _swig_new_static_method(_triangle.Triangle_GetPointMatrix)

    def GetVertices(self, *args):
        r"""
        GetVertices(Triangle self, intArray v)
        GetVertices(Triangle self) -> int *
        """
        return _triangle.Triangle_GetVertices(self, *args)
    GetVertices = _swig_new_instance_method(_triangle.Triangle_GetVertices)

    def GetNVertices(self):
        r"""GetNVertices(Triangle self) -> int"""
        return _triangle.Triangle_GetNVertices(self)
    GetNVertices = _swig_new_instance_method(_triangle.Triangle_GetNVertices)

    def GetNEdges(self):
        r"""GetNEdges(Triangle self) -> int"""
        return _triangle.Triangle_GetNEdges(self)
    GetNEdges = _swig_new_instance_method(_triangle.Triangle_GetNEdges)

    def GetEdgeVertices(self, ei):
        r"""GetEdgeVertices(Triangle self, int ei) -> int const *"""
        return _triangle.Triangle_GetEdgeVertices(self, ei)
    GetEdgeVertices = _swig_new_instance_method(_triangle.Triangle_GetEdgeVertices)

    def GetNFaces(self, *args):
        r"""
        GetNFaces(Triangle self, int & nFaceVertices) -> int
        GetNFaces(Triangle self) -> int
        """

        if len(args) == 1:
             import warnings
             warnings.warn("Triangle::GetNFaces(int & nFaceVertices) is deprecated is deprecated",
         	              DeprecationWarning,)


        return _triangle.Triangle_GetNFaces(self, *args)


    def GetNFaceVertices(self, arg2):
        r"""GetNFaceVertices(Triangle self, int arg2) -> int"""
        return _triangle.Triangle_GetNFaceVertices(self, arg2)
    GetNFaceVertices = _swig_new_instance_method(_triangle.Triangle_GetNFaceVertices)

    def GetFaceVertices(self, fi):
        r"""GetFaceVertices(Triangle self, int fi) -> int const *"""
        return _triangle.Triangle_GetFaceVertices(self, fi)
    GetFaceVertices = _swig_new_instance_method(_triangle.Triangle_GetFaceVertices)

    def Duplicate(self, m):
        r"""Duplicate(Triangle self, mfem::Mesh * m) -> Element"""
        return _triangle.Triangle_Duplicate(self, m)
    Duplicate = _swig_new_instance_method(_triangle.Triangle_Duplicate)
    __swig_destroy__ = _triangle.delete_Triangle

# Register Triangle in _triangle:
_triangle.Triangle_swigregister(Triangle)

def Triangle_GetPointMatrix(transform, pm):
    r"""Triangle_GetPointMatrix(unsigned int transform, DenseMatrix pm)"""
    return _triangle.Triangle_GetPointMatrix(transform, pm)
Triangle_GetPointMatrix = _triangle.Triangle_GetPointMatrix


cvar = _triangle.cvar




# This file was automatically generated by SWIG (http://www.swig.org).
# Version 4.0.2
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.

from sys import version_info as _swig_python_version_info
if _swig_python_version_info < (2, 7, 0):
    raise RuntimeError("Python 2.7 or later required")

# Import the low-level C/C++ module
if __package__ or "." in __name__:
    from . import _nonlinearform
else:
    import _nonlinearform

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

_swig_new_instance_method = _nonlinearform.SWIG_PyInstanceMethod_New
_swig_new_static_method = _nonlinearform.SWIG_PyStaticMethod_New

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "thisown":
            self.this.own(value)
        elif name == "this":
            set(self, name, value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)


import weakref

import mfem._par.operators
import mfem._par.mem_manager
import mfem._par.vector
import mfem._par.array
import mfem._par.fespace
import mfem._par.coefficient
import mfem._par.globals
import mfem._par.matrix
import mfem._par.intrules
import mfem._par.sparsemat
import mfem._par.densemat
import mfem._par.eltrans
import mfem._par.fe
import mfem._par.geom
import mfem._par.mesh
import mfem._par.sort_pairs
import mfem._par.ncmesh
import mfem._par.vtk
import mfem._par.element
import mfem._par.table
import mfem._par.hash
import mfem._par.vertex
import mfem._par.gridfunc
import mfem._par.bilininteg
import mfem._par.fe_coll
import mfem._par.lininteg
import mfem._par.linearform
import mfem._par.handle
import mfem._par.hypre
import mfem._par.restriction
import mfem._par.bilinearform
import mfem._par.nonlininteg
class NonlinearForm(mfem._par.operators.Operator):
    r"""Proxy of C++ mfem::NonlinearForm class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, f):
        r"""__init__(NonlinearForm self, FiniteElementSpace f) -> NonlinearForm"""
        _nonlinearform.NonlinearForm_swiginit(self, _nonlinearform.new_NonlinearForm(f))

    def SetAssemblyLevel(self, assembly_level):
        r"""SetAssemblyLevel(NonlinearForm self, mfem::AssemblyLevel assembly_level)"""
        return _nonlinearform.NonlinearForm_SetAssemblyLevel(self, assembly_level)
    SetAssemblyLevel = _swig_new_instance_method(_nonlinearform.NonlinearForm_SetAssemblyLevel)

    def FESpace(self, *args):
        r"""
        FESpace(NonlinearForm self) -> FiniteElementSpace
        FESpace(NonlinearForm self) -> FiniteElementSpace
        """
        return _nonlinearform.NonlinearForm_FESpace(self, *args)
    FESpace = _swig_new_instance_method(_nonlinearform.NonlinearForm_FESpace)

    def AddDomainIntegrator(self, nlfi):
        r"""AddDomainIntegrator(NonlinearForm self, NonlinearFormIntegrator nlfi)"""

        #    if not hasattr(self, "_integrators"): self._integrators = []
        #    self._integrators.append(nlfi)
        nlfi.thisown=0 


        return _nonlinearform.NonlinearForm_AddDomainIntegrator(self, nlfi)


    def GetDNFI(self, *args):
        r"""
        GetDNFI(NonlinearForm self) -> mfem::Array< mfem::NonlinearFormIntegrator * >
        GetDNFI(NonlinearForm self) -> mfem::Array< mfem::NonlinearFormIntegrator * > const *
        """
        return _nonlinearform.NonlinearForm_GetDNFI(self, *args)
    GetDNFI = _swig_new_instance_method(_nonlinearform.NonlinearForm_GetDNFI)

    def AddInteriorFaceIntegrator(self, nlfi):
        r"""AddInteriorFaceIntegrator(NonlinearForm self, NonlinearFormIntegrator nlfi)"""

        #    if not hasattr(self, "_integrators"): self._integrators = []
        #    self._integrators.append(nlfi)
        nlfi.thisown=0 


        return _nonlinearform.NonlinearForm_AddInteriorFaceIntegrator(self, nlfi)


    def AddBdrFaceIntegrator(self, *args):
        r"""
        AddBdrFaceIntegrator(NonlinearForm self, NonlinearFormIntegrator nlfi)
        AddBdrFaceIntegrator(NonlinearForm self, NonlinearFormIntegrator nfi, intArray bdr_marker)
        """

        #    if not hasattr(self, "_integrators"): self._integrators = []
        #    self._integrators.append(nlfi)
        nlfi = args[0]
        nlfi.thisown=0 


        return _nonlinearform.NonlinearForm_AddBdrFaceIntegrator(self, *args)


    def SetEssentialBC(self, bdr_attr_is_ess, rhs=None):
        r"""SetEssentialBC(NonlinearForm self, intArray bdr_attr_is_ess, Vector rhs=None)"""
        return _nonlinearform.NonlinearForm_SetEssentialBC(self, bdr_attr_is_ess, rhs)
    SetEssentialBC = _swig_new_instance_method(_nonlinearform.NonlinearForm_SetEssentialBC)

    def SetEssentialVDofs(self, ess_vdofs_list):
        r"""SetEssentialVDofs(NonlinearForm self, intArray ess_vdofs_list)"""
        return _nonlinearform.NonlinearForm_SetEssentialVDofs(self, ess_vdofs_list)
    SetEssentialVDofs = _swig_new_instance_method(_nonlinearform.NonlinearForm_SetEssentialVDofs)

    def SetEssentialTrueDofs(self, ess_tdof_list):
        r"""SetEssentialTrueDofs(NonlinearForm self, intArray ess_tdof_list)"""
        return _nonlinearform.NonlinearForm_SetEssentialTrueDofs(self, ess_tdof_list)
    SetEssentialTrueDofs = _swig_new_instance_method(_nonlinearform.NonlinearForm_SetEssentialTrueDofs)

    def GetEssentialTrueDofs(self):
        r"""GetEssentialTrueDofs(NonlinearForm self) -> intArray"""
        return _nonlinearform.NonlinearForm_GetEssentialTrueDofs(self)
    GetEssentialTrueDofs = _swig_new_instance_method(_nonlinearform.NonlinearForm_GetEssentialTrueDofs)

    def GetGridFunctionEnergy(self, x):
        r"""GetGridFunctionEnergy(NonlinearForm self, Vector x) -> double"""
        return _nonlinearform.NonlinearForm_GetGridFunctionEnergy(self, x)
    GetGridFunctionEnergy = _swig_new_instance_method(_nonlinearform.NonlinearForm_GetGridFunctionEnergy)

    def GetEnergy(self, x):
        r"""GetEnergy(NonlinearForm self, Vector x) -> double"""
        return _nonlinearform.NonlinearForm_GetEnergy(self, x)
    GetEnergy = _swig_new_instance_method(_nonlinearform.NonlinearForm_GetEnergy)

    def Mult(self, x, y):
        r"""Mult(NonlinearForm self, Vector x, Vector y)"""
        return _nonlinearform.NonlinearForm_Mult(self, x, y)
    Mult = _swig_new_instance_method(_nonlinearform.NonlinearForm_Mult)

    def GetGradient(self, x):
        r"""GetGradient(NonlinearForm self, Vector x) -> Operator"""
        return _nonlinearform.NonlinearForm_GetGradient(self, x)
    GetGradient = _swig_new_instance_method(_nonlinearform.NonlinearForm_GetGradient)

    def Update(self):
        r"""Update(NonlinearForm self)"""
        return _nonlinearform.NonlinearForm_Update(self)
    Update = _swig_new_instance_method(_nonlinearform.NonlinearForm_Update)

    def Setup(self):
        r"""Setup(NonlinearForm self)"""
        return _nonlinearform.NonlinearForm_Setup(self)
    Setup = _swig_new_instance_method(_nonlinearform.NonlinearForm_Setup)

    def GetProlongation(self):
        r"""GetProlongation(NonlinearForm self) -> Operator"""
        return _nonlinearform.NonlinearForm_GetProlongation(self)
    GetProlongation = _swig_new_instance_method(_nonlinearform.NonlinearForm_GetProlongation)

    def GetRestriction(self):
        r"""GetRestriction(NonlinearForm self) -> Operator"""
        return _nonlinearform.NonlinearForm_GetRestriction(self)
    GetRestriction = _swig_new_instance_method(_nonlinearform.NonlinearForm_GetRestriction)
    __swig_destroy__ = _nonlinearform.delete_NonlinearForm

    def GetGradientMatrix(self, x):
        r"""GetGradientMatrix(NonlinearForm self, Vector x) -> SparseMatrix"""
        return _nonlinearform.NonlinearForm_GetGradientMatrix(self, x)
    GetGradientMatrix = _swig_new_instance_method(_nonlinearform.NonlinearForm_GetGradientMatrix)

# Register NonlinearForm in _nonlinearform:
_nonlinearform.NonlinearForm_swigregister(NonlinearForm)

class BlockNonlinearForm(mfem._par.operators.Operator):
    r"""Proxy of C++ mfem::BlockNonlinearForm class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(BlockNonlinearForm self) -> BlockNonlinearForm
        __init__(BlockNonlinearForm self, mfem::Array< mfem::FiniteElementSpace * > & f) -> BlockNonlinearForm
        """
        _nonlinearform.BlockNonlinearForm_swiginit(self, _nonlinearform.new_BlockNonlinearForm(*args))

    def FESpace(self, *args):
        r"""
        FESpace(BlockNonlinearForm self, int k) -> FiniteElementSpace
        FESpace(BlockNonlinearForm self, int k) -> FiniteElementSpace
        """
        return _nonlinearform.BlockNonlinearForm_FESpace(self, *args)
    FESpace = _swig_new_instance_method(_nonlinearform.BlockNonlinearForm_FESpace)

    def SetSpaces(self, f):
        r"""SetSpaces(BlockNonlinearForm self, mfem::Array< mfem::FiniteElementSpace * > & f)"""
        return _nonlinearform.BlockNonlinearForm_SetSpaces(self, f)
    SetSpaces = _swig_new_instance_method(_nonlinearform.BlockNonlinearForm_SetSpaces)

    def GetBlockOffsets(self):
        r"""GetBlockOffsets(BlockNonlinearForm self) -> intArray"""
        return _nonlinearform.BlockNonlinearForm_GetBlockOffsets(self)
    GetBlockOffsets = _swig_new_instance_method(_nonlinearform.BlockNonlinearForm_GetBlockOffsets)

    def GetBlockTrueOffsets(self):
        r"""GetBlockTrueOffsets(BlockNonlinearForm self) -> intArray"""
        return _nonlinearform.BlockNonlinearForm_GetBlockTrueOffsets(self)
    GetBlockTrueOffsets = _swig_new_instance_method(_nonlinearform.BlockNonlinearForm_GetBlockTrueOffsets)

    def AddDomainIntegrator(self, nlfi):
        r"""AddDomainIntegrator(BlockNonlinearForm self, BlockNonlinearFormIntegrator nlfi)"""

        #    if not hasattr(self, "_integrators"): self._integrators = []
        #    self._integrators.append(nlfi)
        nlfi.thisown=0 


        return _nonlinearform.BlockNonlinearForm_AddDomainIntegrator(self, nlfi)


    def AddInteriorFaceIntegrator(self, nlfi):
        r"""AddInteriorFaceIntegrator(BlockNonlinearForm self, BlockNonlinearFormIntegrator nlfi)"""

        #    if not hasattr(self, "_integrators"): self._integrators = []
        #    self._integrators.append(nlfi)
        nlfi.thisown=0 


        return _nonlinearform.BlockNonlinearForm_AddInteriorFaceIntegrator(self, nlfi)


    def AddBdrFaceIntegrator(self, *args):
        r"""
        AddBdrFaceIntegrator(BlockNonlinearForm self, BlockNonlinearFormIntegrator nlfi)
        AddBdrFaceIntegrator(BlockNonlinearForm self, BlockNonlinearFormIntegrator nlfi, intArray bdr_marker)
        """

        #    if not hasattr(self, "_integrators"): self._integrators = []
        #    self._integrators.append(nlfi)
        nlfi = args[0]
        nlfi.thisown=0 


        return _nonlinearform.BlockNonlinearForm_AddBdrFaceIntegrator(self, *args)


    def SetEssentialBC(self, bdr_attr_is_ess, rhs):
        r"""SetEssentialBC(BlockNonlinearForm self, mfem::Array< mfem::Array< int > * > const & bdr_attr_is_ess, mfem::Array< mfem::Vector * > & rhs)"""
        return _nonlinearform.BlockNonlinearForm_SetEssentialBC(self, bdr_attr_is_ess, rhs)
    SetEssentialBC = _swig_new_instance_method(_nonlinearform.BlockNonlinearForm_SetEssentialBC)

    def GetEnergy(self, x):
        r"""GetEnergy(BlockNonlinearForm self, Vector x) -> double"""
        return _nonlinearform.BlockNonlinearForm_GetEnergy(self, x)
    GetEnergy = _swig_new_instance_method(_nonlinearform.BlockNonlinearForm_GetEnergy)

    def Mult(self, x, y):
        r"""Mult(BlockNonlinearForm self, Vector x, Vector y)"""
        return _nonlinearform.BlockNonlinearForm_Mult(self, x, y)
    Mult = _swig_new_instance_method(_nonlinearform.BlockNonlinearForm_Mult)

    def GetGradient(self, x):
        r"""GetGradient(BlockNonlinearForm self, Vector x) -> Operator"""
        return _nonlinearform.BlockNonlinearForm_GetGradient(self, x)
    GetGradient = _swig_new_instance_method(_nonlinearform.BlockNonlinearForm_GetGradient)
    __swig_destroy__ = _nonlinearform.delete_BlockNonlinearForm

# Register BlockNonlinearForm in _nonlinearform:
_nonlinearform.BlockNonlinearForm_swigregister(BlockNonlinearForm)






'''
   MFEM example 2

   See c++ version in the MFEM library for more detail 
'''
from mfem import path
import mfem.ser as mfem
from mfem.ser import intArray
from os.path import expanduser, join
import numpy as np

order = 1
visualization = True
meshfile = expanduser(join(path, 'data', 'beam-tri.mesh'))
mesh = mfem.Mesh(meshfile, 1,1)
dim = mesh.Dimension()

#   3. Refine the mesh to increase the resolution. In this example we do
#      'ref_levels' of uniform refinement. We choose 'ref_levels' to be the
#      largest number that gives a final mesh with no more than 50,000
#      elements.

ref_levels = int(np.floor(np.log(5000./mesh.GetNE())/np.log(2.)/dim))
for x in range(ref_levels):
   mesh.UniformRefinement();

#5. Define a finite element space on the mesh. Here we use vector finite
#   elements, i.e. dim copies of a scalar finite element space. The vector
#   dimension is specified by the last argument of the FiniteElementSpace
#   constructor. For NURBS meshes, we use the (degree elevated) NURBS space
#   associated with the mesh nodes.
fec = mfem.H1_FECollection(order, dim)
fespace = mfem.FiniteElementSpace(mesh, fec, dim)

print("Number of finite element unknowns: " + str(fespace.GetTrueVSize()))

# 6. Determine the list of true (i.e. conforming) essential boundary dofs.
#    In this example, the boundary conditions are defined by marking only
#    boundary attribute 1 from the mesh as essential and converting it to a
#    list of true dofs. 
ess_tdof_list = intArray()
ess_bdr = intArray([1]+[0]*(mesh.bdr_attributes.Max()-1))
fespace.GetEssentialTrueDofs(ess_bdr, ess_tdof_list)
# 7. Set up the linear form b(.) which corresponds to the right-hand side of
#    the FEM linear system. In this case, b_i equals the boundary integral
#    of f*phi_i where f represents a "pull down" force on the Neumann part
#    of the boundary and phi_i are the basis functions in the finite element
#    fespace. The force is defined by the VectorArrayCoefficient object f,
#    which is a vector of Coefficient objects. The fact that f is non-zero
#    on boundary attribute 2 is indicated by the use of piece-wise constants
#    coefficient for its last component.
f = mfem.VectorArrayCoefficient(dim)
for i in range(dim-1): f.Set(i, mfem.ConstantCoefficient(0.0)); 

pull_force = mfem.Vector([0]*mesh.bdr_attributes.Max())
pull_force[1] = -1.0e-2;
f.Set(dim-1, mfem.PWConstCoefficient(pull_force))

b = mfem.LinearForm(fespace)
b.AddBoundaryIntegrator(mfem.VectorBoundaryLFIntegrator(f))
print('r.h.s...')
b.Assemble()
# 8. Define the solution vector x as a finite element grid function
#    corresponding to fespace. Initialize x with initial guess of zero,
#    which satisfies the boundary conditions.
x = mfem.GridFunction(fespace)
x.Assign(0.0)  
# 9. Set up the bilinear form a(.,.) on the finite element space
#    corresponding to the linear elasticity integrator with piece-wise
#    constants coefficient lambda and mu.
#    (Here lambda is renamed by lamb since it is reserved in python)
lamb = mfem.Vector(mesh.attributes.Max())
lamb.Assign(1.0)
lamb[0] = lamb[1]*50;
lambda_func = mfem.PWConstCoefficient(lamb)
mu = mfem.Vector(mesh.attributes.Max())
mu.Assign(1.0);
mu[0] = mu[1]*50;
mu_func = mfem.PWConstCoefficient(mu)
a = mfem.BilinearForm(fespace)
a.AddDomainIntegrator(mfem.ElasticityIntegrator(lambda_func,mu_func))
# 10. Assemble the bilinear form and the corresponding linear system,
#     applying any necessary transformations such as: eliminating boundary
#     conditions, applying conforming constraints for non-conforming AMR,
#     static condensation, etc.
print('matrix...')
static_cond = False
if (static_cond): a.EnableStaticCondensation()
a.Assemble()


A = mfem.OperatorPtr()
B = mfem.Vector()
X = mfem.Vector()
a.FormLinearSystem(ess_tdof_list, x, b, A, X, B);
print('...done')## Here, original version calls hegith, which is not
## defined in the header...!?
print("Size of linear system: " + str(A.Height())) 

# 10. Solve
AA = mfem.OperatorHandle2SparseMatrix(A)
M = mfem.GSSmoother(AA)
mfem.PCG(AA, M, B, X, 1, 500, 1e-8, 0.0);

# 11. Recover the solution as a finite element grid function.
a.RecoverFEMSolution(X, b, x);

# 13. For non-NURBS meshes, make the mesh curved based on the finite element
#     space. This means that we define the mesh elements through a fespace
#     based transformation of the reference element. This allows us to save
#     the displaced mesh as a curved mesh when using high-order finite
#     element displacement field. We assume that the initial mesh (read from
#     the file) is not higher order curved mesh compared to the chosen FE
#     space.
if not mesh.NURBSext:
     mesh.SetNodalFESpace(fespace)


# 14. Save the displaced mesh and the inverted solution (which gives the
#     backward displacements to the original grid). This output can be
#     viewed later using GLVis: "glvis -m displaced.mesh -g sol.gf".
nodes = mesh.GetNodes()
nodes += x
x *= -1
mesh.Print('displaced.mesh', 8)
x.Save('sol.gf', 8)


if (visualization):
    sol_sock = mfem.socketstream("localhost", 19916)
    sol_sock.precision(8)
    sol_sock.send_solution(mesh,  x)




'''
   MFEM example 1p

   See c++ version in the MFEM library for more detail 
'''
import sys
import os
from os.path import expanduser, join
import numpy as np

from mpi4py import MPI

from mfem.common.arg_parser import ArgParser
from mfem import path
import mfem.par as mfem

def_meshfile = expanduser(join(path, 'data', 'star.mesh'))

num_proc = MPI.COMM_WORLD.size
myid     = MPI.COMM_WORLD.rank

def run(order = 1, static_cond = False,
        meshfile = def_meshfile, visualization = False,
        use_strumpack = False):

   mesh = mfem.Mesh(meshfile, 1,1)
   dim = mesh.Dimension()

   ref_levels = int(np.floor(np.log(10000./mesh.GetNE())/np.log(2.)/dim))
   for x in range(ref_levels):
      mesh.UniformRefinement();
   mesh.ReorientTetMesh();
   pmesh = mfem.ParMesh(MPI.COMM_WORLD, mesh)
   del mesh

   par_ref_levels = 2
   for l in range(par_ref_levels):
       pmesh.UniformRefinement();

   if order > 0:
       fec = mfem.H1_FECollection(order, dim)
   elif mesh.GetNodes():
       fec = mesh.GetNodes().OwnFEC()
       print( "Using isoparametric FEs: " + str(fec.Name()));
   else:
       order = 1
       fec = mfem.H1_FECollection(order, dim)

   fespace =mfem.ParFiniteElementSpace(pmesh, fec)
   fe_size = fespace.GlobalTrueVSize()

   if (myid == 0):
      print('Number of finite element unknowns: '+  str(fe_size))

   ess_tdof_list = mfem.intArray()
   if pmesh.bdr_attributes.Size()>0:
       ess_bdr = mfem.intArray(pmesh.bdr_attributes.Max())
       ess_bdr.Assign(1)
       fespace.GetEssentialTrueDofs(ess_bdr, ess_tdof_list)

   #   the basis functions in the finite element fespace.
   b = mfem.ParLinearForm(fespace)
   one = mfem.ConstantCoefficient(1.0)
   b.AddDomainIntegrator(mfem.DomainLFIntegrator(one))
   b.Assemble();

   x = mfem.ParGridFunction(fespace);
   x.Assign(0.0)

   a = mfem.ParBilinearForm(fespace);
   a.AddDomainIntegrator(mfem.DiffusionIntegrator(one))

   if static_cond: a.EnableStaticCondensation()
   a.Assemble();

   A = mfem.HypreParMatrix()
   B = mfem.Vector()
   X = mfem.Vector()
   a.FormLinearSystem(ess_tdof_list, x, b, A, X, B)

   if (myid == 0):
      print("Size of linear system: " + str(x.Size()))
      print("Size of linear system: " + str(A.GetGlobalNumRows()))

   if use_strumpack:
       import mfem.par.strumpack as strmpk
       Arow = strmpk.STRUMPACKRowLocMatrix(A)
       args = ["--sp_hss_min_sep_size", "128", "--sp_enable_hss"]
       strumpack = strmpk.STRUMPACKSolver(args, MPI.COMM_WORLD)
       strumpack.SetPrintFactorStatistics(True)
       strumpack.SetPrintSolveStatistics(False)
       strumpack.SetKrylovSolver(strmpk.KrylovSolver_DIRECT);
       strumpack.SetReorderingStrategy(strmpk.ReorderingStrategy_METIS)
       strumpack.SetMC64Job(strmpk.MC64Job_NONE)
       # strumpack.SetSymmetricPattern(True)
       strumpack.SetOperator(Arow)
       strumpack.SetFromCommandLine()
       strumpack.Mult(B, X);

   else:
       amg = mfem.HypreBoomerAMG(A)
       cg = mfem.CGSolver(MPI.COMM_WORLD)
       cg.SetRelTol(1e-12)
       cg.SetMaxIter(200)
       cg.SetPrintLevel(1)
       cg.SetPreconditioner(amg)
       cg.SetOperator(A)
       cg.Mult(B, X);


   a.RecoverFEMSolution(X, b, x)

   smyid = '{:0>6d}'.format(myid)
   mesh_name  =  "mesh."+smyid
   sol_name   =  "sol."+smyid

   pmesh.Print(mesh_name, 8)
   x.Save(sol_name, 8)

if __name__ == "__main__":
   parser = ArgParser(description='Ex1 (Laplace Problem)')
   parser.add_argument('-m', '--mesh',
                       default = 'star.mesh',
                       action = 'store', type = str,
                       help='Mesh file to use.')
   parser.add_argument('-vis', '--visualization',
                       action = 'store_true',
                       help='Enable GLVis visualization')
   parser.add_argument('-o', '--order',
                       action = 'store', default = 1, type=int,
                       help = "Finite element order (polynomial degree) or -1 for isoparametric space.");
   parser.add_argument('-sc', '--static-condensation',
                       action = 'store_true', 
                       help = "Enable static condensation.")
   parser.add_argument("-sp", "--strumpack",
                   action = 'store_true', default = False,
                   help =  "Use the STRUMPACK Solver.")
   
   args = parser.parse_args()

   if myid == 0: parser.print_options(args)

   order = args.order
   static_cond = args.static_condensation
   meshfile = expanduser(join(os.path.dirname(__file__), '..', 'data', args.mesh))   
   visualization = args.visualization
   use_strumpack = args.strumpack
   run(order = order, static_cond = static_cond,
       meshfile = meshfile, visualization = visualization,
       use_strumpack = use_strumpack)
   







# This file was automatically generated by SWIG (http://www.swig.org).
# Version 3.0.12
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.

from sys import version_info as _swig_python_version_info
if _swig_python_version_info >= (2, 7, 0):
    def swig_import_helper():
        import importlib
        pkg = __name__.rpartition('.')[0]
        mname = '.'.join((pkg, '_strumpack')).lstrip('.')
        try:
            return importlib.import_module(mname)
        except ImportError:
            return importlib.import_module('_strumpack')
    _strumpack = swig_import_helper()
    del swig_import_helper
elif _swig_python_version_info >= (2, 6, 0):
    def swig_import_helper():
        from os.path import dirname
        import imp
        fp = None
        try:
            fp, pathname, description = imp.find_module('_strumpack', [dirname(__file__)])
        except ImportError:
            import _strumpack
            return _strumpack
        try:
            _mod = imp.load_module('_strumpack', fp, pathname, description)
        finally:
            if fp is not None:
                fp.close()
        return _mod
    _strumpack = swig_import_helper()
    del swig_import_helper
else:
    import _strumpack
del _swig_python_version_info

try:
    _swig_property = property
except NameError:
    pass  # Python < 2.2 doesn't have 'property'.

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

def _swig_setattr_nondynamic(self, class_type, name, value, static=1):
    if (name == "thisown"):
        return self.this.own(value)
    if (name == "this"):
        if type(value).__name__ == 'SwigPyObject':
            self.__dict__[name] = value
            return
    method = class_type.__swig_setmethods__.get(name, None)
    if method:
        return method(self, value)
    if (not static):
        if _newclass:
            object.__setattr__(self, name, value)
        else:
            self.__dict__[name] = value
    else:
        raise AttributeError("You cannot add attributes to %s" % self)


def _swig_setattr(self, class_type, name, value):
    return _swig_setattr_nondynamic(self, class_type, name, value, 0)


def _swig_getattr(self, class_type, name):
    if (name == "thisown"):
        return self.this.own()
    method = class_type.__swig_getmethods__.get(name, None)
    if method:
        return method(self)
    raise AttributeError("'%s' object has no attribute '%s'" % (class_type.__name__, name))


def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)

try:
    _object = object
    _newclass = 1
except __builtin__.Exception:
    class _object:
        pass
    _newclass = 0

try:
    import weakref
    weakref_proxy = weakref.proxy
except __builtin__.Exception:
    weakref_proxy = lambda x: x


MFEM_VERSION = _strumpack.MFEM_VERSION
MFEM_VERSION_STRING = _strumpack.MFEM_VERSION_STRING
MFEM_VERSION_TYPE = _strumpack.MFEM_VERSION_TYPE
MFEM_VERSION_TYPE_RELEASE = _strumpack.MFEM_VERSION_TYPE_RELEASE
MFEM_VERSION_TYPE_DEVELOPMENT = _strumpack.MFEM_VERSION_TYPE_DEVELOPMENT
MFEM_VERSION_MAJOR = _strumpack.MFEM_VERSION_MAJOR
MFEM_VERSION_MINOR = _strumpack.MFEM_VERSION_MINOR
MFEM_VERSION_PATCH = _strumpack.MFEM_VERSION_PATCH
MFEM_TIMER_TYPE = _strumpack.MFEM_TIMER_TYPE
MFEM_HYPRE_VERSION = _strumpack.MFEM_HYPRE_VERSION
class intp(_object):
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, intp, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, intp, name)
    __repr__ = _swig_repr

    def __init__(self):
        this = _strumpack.new_intp()
        try:
            self.this.append(this)
        except __builtin__.Exception:
            self.this = this
    __swig_destroy__ = _strumpack.delete_intp
    __del__ = lambda self: None

    def assign(self, value):
        return _strumpack.intp_assign(self, value)

    def value(self):
        return _strumpack.intp_value(self)

    def cast(self):
        return _strumpack.intp_cast(self)
    if _newclass:
        frompointer = staticmethod(_strumpack.intp_frompointer)
    else:
        frompointer = _strumpack.intp_frompointer
intp_swigregister = _strumpack.intp_swigregister
intp_swigregister(intp)

def intp_frompointer(t):
    return _strumpack.intp_frompointer(t)
intp_frompointer = _strumpack.intp_frompointer

class doublep(_object):
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, doublep, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, doublep, name)
    __repr__ = _swig_repr

    def __init__(self):
        this = _strumpack.new_doublep()
        try:
            self.this.append(this)
        except __builtin__.Exception:
            self.this = this
    __swig_destroy__ = _strumpack.delete_doublep
    __del__ = lambda self: None

    def assign(self, value):
        return _strumpack.doublep_assign(self, value)

    def value(self):
        return _strumpack.doublep_value(self)

    def cast(self):
        return _strumpack.doublep_cast(self)
    if _newclass:
        frompointer = staticmethod(_strumpack.doublep_frompointer)
    else:
        frompointer = _strumpack.doublep_frompointer
doublep_swigregister = _strumpack.doublep_swigregister
doublep_swigregister(doublep)

def doublep_frompointer(t):
    return _strumpack.doublep_frompointer(t)
doublep_frompointer = _strumpack.doublep_frompointer

import ostream_typemap
import operators
import vector
import array
import hypre
import sparsemat
import matrix
import densemat
import fespace
import coefficient
import intrules
import eltrans
import fe
import mesh
import ncmesh
import element
import geom
import table
import vertex
import gridfunc
import bilininteg
import fe_coll
import lininteg
import linearform
import handle
import communication
import sets

def argv_obj(input):
    """argv_obj(PyObject * input) -> char **"""
    return _strumpack.argv_obj(input)
ReorderingStrategy_NATURAL = _strumpack.ReorderingStrategy_NATURAL
ReorderingStrategy_METIS = _strumpack.ReorderingStrategy_METIS
ReorderingStrategy_PARMETIS = _strumpack.ReorderingStrategy_PARMETIS
ReorderingStrategy_SCOTCH = _strumpack.ReorderingStrategy_SCOTCH
ReorderingStrategy_PTSCOTCH = _strumpack.ReorderingStrategy_PTSCOTCH
ReorderingStrategy_RCM = _strumpack.ReorderingStrategy_RCM
ReorderingStrategy_GEOMETRIC = _strumpack.ReorderingStrategy_GEOMETRIC
MC64Job_NONE = _strumpack.MC64Job_NONE
MC64Job_MAX_CARDINALITY = _strumpack.MC64Job_MAX_CARDINALITY
MC64Job_MAX_SMALLEST_DIAGONAL = _strumpack.MC64Job_MAX_SMALLEST_DIAGONAL
MC64Job_MAX_SMALLEST_DIAGONAL_2 = _strumpack.MC64Job_MAX_SMALLEST_DIAGONAL_2
MC64Job_MAX_DIAGONAL_SUM = _strumpack.MC64Job_MAX_DIAGONAL_SUM
MC64Job_MAX_DIAGONAL_PRODUCT_SCALING = _strumpack.MC64Job_MAX_DIAGONAL_PRODUCT_SCALING

def MC64_job_number(job):
    """MC64_job_number(strumpack::MC64Job job) -> int"""
    return _strumpack.MC64_job_number(job)

def get_name(method):
    """get_name(strumpack::ReorderingStrategy method) -> std::string"""
    return _strumpack.get_name(method)

def is_parallel(method):
    """is_parallel(strumpack::ReorderingStrategy method) -> bool"""
    return _strumpack.is_parallel(method)
GramSchmidtType_CLASSICAL = _strumpack.GramSchmidtType_CLASSICAL
GramSchmidtType_MODIFIED = _strumpack.GramSchmidtType_MODIFIED
KrylovSolver_AUTO = _strumpack.KrylovSolver_AUTO
KrylovSolver_DIRECT = _strumpack.KrylovSolver_DIRECT
KrylovSolver_REFINE = _strumpack.KrylovSolver_REFINE
KrylovSolver_PREC_GMRES = _strumpack.KrylovSolver_PREC_GMRES
KrylovSolver_GMRES = _strumpack.KrylovSolver_GMRES
KrylovSolver_PREC_BICGSTAB = _strumpack.KrylovSolver_PREC_BICGSTAB
KrylovSolver_BICGSTAB = _strumpack.KrylovSolver_BICGSTAB
class STRUMPACKRowLocMatrix(operators.Operator):
    """Proxy of C++ mfem::STRUMPACKRowLocMatrix class."""

    __swig_setmethods__ = {}
    for _s in [operators.Operator]:
        __swig_setmethods__.update(getattr(_s, '__swig_setmethods__', {}))
    __setattr__ = lambda self, name, value: _swig_setattr(self, STRUMPACKRowLocMatrix, name, value)
    __swig_getmethods__ = {}
    for _s in [operators.Operator]:
        __swig_getmethods__.update(getattr(_s, '__swig_getmethods__', {}))
    __getattr__ = lambda self, name: _swig_getattr(self, STRUMPACKRowLocMatrix, name)
    __repr__ = _swig_repr

    def __init__(self, *args):
        """
        __init__(mfem::STRUMPACKRowLocMatrix self, MPI_Comm comm, int num_loc_rows, int first_loc_row, int glob_nrows, int glob_ncols, int * I, int * J, double * data) -> STRUMPACKRowLocMatrix
        __init__(mfem::STRUMPACKRowLocMatrix self, HypreParMatrix hypParMat) -> STRUMPACKRowLocMatrix
        """
        this = _strumpack.new_STRUMPACKRowLocMatrix(*args)
        try:
            self.this.append(this)
        except __builtin__.Exception:
            self.this = this
    __swig_destroy__ = _strumpack.delete_STRUMPACKRowLocMatrix
    __del__ = lambda self: None

    def Mult(self, x, y):
        """Mult(STRUMPACKRowLocMatrix self, Vector x, Vector y)"""
        return _strumpack.STRUMPACKRowLocMatrix_Mult(self, x, y)


    def GetComm(self):
        """GetComm(STRUMPACKRowLocMatrix self) -> MPI_Comm"""
        return _strumpack.STRUMPACKRowLocMatrix_GetComm(self)


    def getA(self):
        """getA(STRUMPACKRowLocMatrix self) -> strumpack::CSRMatrixMPI< double,int > *"""
        return _strumpack.STRUMPACKRowLocMatrix_getA(self)

STRUMPACKRowLocMatrix_swigregister = _strumpack.STRUMPACKRowLocMatrix_swigregister
STRUMPACKRowLocMatrix_swigregister(STRUMPACKRowLocMatrix)

class STRUMPACKSolver(operators.Solver):
    """Proxy of C++ mfem::STRUMPACKSolver class."""

    __swig_setmethods__ = {}
    for _s in [operators.Solver]:
        __swig_setmethods__.update(getattr(_s, '__swig_setmethods__', {}))
    __setattr__ = lambda self, name, value: _swig_setattr(self, STRUMPACKSolver, name, value)
    __swig_getmethods__ = {}
    for _s in [operators.Solver]:
        __swig_getmethods__.update(getattr(_s, '__swig_getmethods__', {}))
    __getattr__ = lambda self, name: _swig_getattr(self, STRUMPACKSolver, name)
    __repr__ = _swig_repr

    def __init__(self, *args):
        """
        __init__(mfem::STRUMPACKSolver self, int argc, char *[] argv, MPI_Comm comm) -> STRUMPACKSolver
        __init__(mfem::STRUMPACKSolver self, STRUMPACKRowLocMatrix A) -> STRUMPACKSolver
        """

        attach_argv = False
        if isinstance(args[0], list):
            aa = [""]+args[0]  
            num = len(aa)
            ptr = argv_obj(aa)
            args = (num, ptr, args[1])
            attach_argv = True



        this = _strumpack.new_STRUMPACKSolver(*args)
        try:
            self.this.append(this)
        except __builtin__.Exception:
            self.this = this

        if attach_argv:
            self._argv = ptr



    __swig_destroy__ = _strumpack.delete_STRUMPACKSolver
    __del__ = lambda self: None

    def Mult(self, x, y):
        """Mult(STRUMPACKSolver self, Vector x, Vector y)"""
        return _strumpack.STRUMPACKSolver_Mult(self, x, y)


    def SetOperator(self, op):
        """SetOperator(STRUMPACKSolver self, Operator op)"""
        return _strumpack.STRUMPACKSolver_SetOperator(self, op)


    def SetFromCommandLine(self):
        """SetFromCommandLine(STRUMPACKSolver self)"""
        return _strumpack.STRUMPACKSolver_SetFromCommandLine(self)


    def SetPrintFactorStatistics(self, print_stat):
        """SetPrintFactorStatistics(STRUMPACKSolver self, bool print_stat)"""
        return _strumpack.STRUMPACKSolver_SetPrintFactorStatistics(self, print_stat)


    def SetPrintSolveStatistics(self, print_stat):
        """SetPrintSolveStatistics(STRUMPACKSolver self, bool print_stat)"""
        return _strumpack.STRUMPACKSolver_SetPrintSolveStatistics(self, print_stat)


    def SetRelTol(self, rtol):
        """SetRelTol(STRUMPACKSolver self, double rtol)"""
        return _strumpack.STRUMPACKSolver_SetRelTol(self, rtol)


    def SetAbsTol(self, atol):
        """SetAbsTol(STRUMPACKSolver self, double atol)"""
        return _strumpack.STRUMPACKSolver_SetAbsTol(self, atol)


    def SetKrylovSolver(self, method):
        """SetKrylovSolver(STRUMPACKSolver self, strumpack::KrylovSolver method)"""
        return _strumpack.STRUMPACKSolver_SetKrylovSolver(self, method)


    def SetReorderingStrategy(self, method):
        """SetReorderingStrategy(STRUMPACKSolver self, strumpack::ReorderingStrategy method)"""
        return _strumpack.STRUMPACKSolver_SetReorderingStrategy(self, method)


    def SetMC64Job(self, job):
        """SetMC64Job(STRUMPACKSolver self, strumpack::MC64Job job)"""
        return _strumpack.STRUMPACKSolver_SetMC64Job(self, job)

STRUMPACKSolver_swigregister = _strumpack.STRUMPACKSolver_swigregister
STRUMPACKSolver_swigregister(STRUMPACKSolver)

# This file is compatible with both classic and new-style classes.





# This file was automatically generated by SWIG (http://www.swig.org).
# Version 4.0.2
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.

from sys import version_info as _swig_python_version_info
if _swig_python_version_info < (2, 7, 0):
    raise RuntimeError("Python 2.7 or later required")

# Import the low-level C/C++ module
if __package__ or "." in __name__:
    from . import _sparsesmoothers
else:
    import _sparsesmoothers

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

_swig_new_instance_method = _sparsesmoothers.SWIG_PyInstanceMethod_New
_swig_new_static_method = _sparsesmoothers.SWIG_PyStaticMethod_New

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "thisown":
            self.this.own(value)
        elif name == "this":
            set(self, name, value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)


import weakref

import mfem._par.vector
import mfem._par.array
import mfem._par.mem_manager
import mfem._par.operators
import mfem._par.sparsemat
import mfem._par.matrix
import mfem._par.densemat
class SparseSmoother(mfem._par.matrix.MatrixInverse):
    r"""Proxy of C++ mfem::SparseSmoother class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined - class is abstract")
    __repr__ = _swig_repr

    def SetOperator(self, a):
        r"""SetOperator(SparseSmoother self, Operator a)"""
        return _sparsesmoothers.SparseSmoother_SetOperator(self, a)
    SetOperator = _swig_new_instance_method(_sparsesmoothers.SparseSmoother_SetOperator)
    __swig_destroy__ = _sparsesmoothers.delete_SparseSmoother

# Register SparseSmoother in _sparsesmoothers:
_sparsesmoothers.SparseSmoother_swigregister(SparseSmoother)

class GSSmoother(SparseSmoother):
    r"""Proxy of C++ mfem::GSSmoother class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(GSSmoother self, int t=0, int it=1) -> GSSmoother
        __init__(GSSmoother self, SparseMatrix a, int t=0, int it=1) -> GSSmoother
        """
        _sparsesmoothers.GSSmoother_swiginit(self, _sparsesmoothers.new_GSSmoother(*args))

    def Mult(self, x, y):
        r"""Mult(GSSmoother self, Vector x, Vector y)"""
        return _sparsesmoothers.GSSmoother_Mult(self, x, y)
    Mult = _swig_new_instance_method(_sparsesmoothers.GSSmoother_Mult)
    __swig_destroy__ = _sparsesmoothers.delete_GSSmoother

# Register GSSmoother in _sparsesmoothers:
_sparsesmoothers.GSSmoother_swigregister(GSSmoother)

class DSmoother(SparseSmoother):
    r"""Proxy of C++ mfem::DSmoother class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(DSmoother self, int t=0, double s=1., int it=1) -> DSmoother
        __init__(DSmoother self, SparseMatrix a, int t=0, double s=1., int it=1) -> DSmoother
        """
        _sparsesmoothers.DSmoother_swiginit(self, _sparsesmoothers.new_DSmoother(*args))

    def Mult(self, x, y):
        r"""Mult(DSmoother self, Vector x, Vector y)"""
        return _sparsesmoothers.DSmoother_Mult(self, x, y)
    Mult = _swig_new_instance_method(_sparsesmoothers.DSmoother_Mult)
    __swig_destroy__ = _sparsesmoothers.delete_DSmoother

# Register DSmoother in _sparsesmoothers:
_sparsesmoothers.DSmoother_swigregister(DSmoother)






# This file was automatically generated by SWIG (http://www.swig.org).
# Version 4.0.2
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.

from sys import version_info as _swig_python_version_info
if _swig_python_version_info < (2, 7, 0):
    raise RuntimeError("Python 2.7 or later required")

# Import the low-level C/C++ module
if __package__ or "." in __name__:
    from . import _matrix
else:
    import _matrix

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

_swig_new_instance_method = _matrix.SWIG_PyInstanceMethod_New
_swig_new_static_method = _matrix.SWIG_PyStaticMethod_New

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "thisown":
            self.this.own(value)
        elif name == "this":
            set(self, name, value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)


import weakref

import mfem._par.vector
import mfem._par.array
import mfem._par.mem_manager
import mfem._par.operators
class Matrix(mfem._par.operators.Operator):
    r"""Proxy of C++ mfem::Matrix class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined - class is abstract")
    __repr__ = _swig_repr

    def IsSquare(self):
        r"""IsSquare(Matrix self) -> bool"""
        return _matrix.Matrix_IsSquare(self)
    IsSquare = _swig_new_instance_method(_matrix.Matrix_IsSquare)

    def Elem(self, *args):
        r"""
        Elem(Matrix self, int i, int j) -> double
        Elem(Matrix self, int i, int j) -> double const &
        """
        return _matrix.Matrix_Elem(self, *args)
    Elem = _swig_new_instance_method(_matrix.Matrix_Elem)

    def Inverse(self):
        r"""Inverse(Matrix self) -> MatrixInverse"""
        return _matrix.Matrix_Inverse(self)
    Inverse = _swig_new_instance_method(_matrix.Matrix_Inverse)

    def Finalize(self, arg2):
        r"""Finalize(Matrix self, int arg2)"""
        return _matrix.Matrix_Finalize(self, arg2)
    Finalize = _swig_new_instance_method(_matrix.Matrix_Finalize)
    __swig_destroy__ = _matrix.delete_Matrix

    def Print(self, *args):
        r"""
        Print(Matrix self, std::ostream & out=mfem::out, int width_=4)
        Print(Matrix self, char const * file, int precision=8)
        """
        return _matrix.Matrix_Print(self, *args)
    Print = _swig_new_instance_method(_matrix.Matrix_Print)

# Register Matrix in _matrix:
_matrix.Matrix_swigregister(Matrix)

class MatrixInverse(mfem._par.operators.Solver):
    r"""Proxy of C++ mfem::MatrixInverse class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined - class is abstract")
    __repr__ = _swig_repr
    __swig_destroy__ = _matrix.delete_MatrixInverse

# Register MatrixInverse in _matrix:
_matrix.MatrixInverse_swigregister(MatrixInverse)

class AbstractSparseMatrix(Matrix):
    r"""Proxy of C++ mfem::AbstractSparseMatrix class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined - class is abstract")
    __repr__ = _swig_repr

    def NumNonZeroElems(self):
        r"""NumNonZeroElems(AbstractSparseMatrix self) -> int"""
        return _matrix.AbstractSparseMatrix_NumNonZeroElems(self)
    NumNonZeroElems = _swig_new_instance_method(_matrix.AbstractSparseMatrix_NumNonZeroElems)

    def GetRow(self, row, cols, srow):
        r"""GetRow(AbstractSparseMatrix self, int const row, intArray cols, Vector srow) -> int"""
        return _matrix.AbstractSparseMatrix_GetRow(self, row, cols, srow)
    GetRow = _swig_new_instance_method(_matrix.AbstractSparseMatrix_GetRow)

    def EliminateZeroRows(self, threshold=1e-12):
        r"""EliminateZeroRows(AbstractSparseMatrix self, double const threshold=1e-12)"""
        return _matrix.AbstractSparseMatrix_EliminateZeroRows(self, threshold)
    EliminateZeroRows = _swig_new_instance_method(_matrix.AbstractSparseMatrix_EliminateZeroRows)

    def Mult(self, x, y):
        r"""Mult(AbstractSparseMatrix self, Vector x, Vector y)"""
        return _matrix.AbstractSparseMatrix_Mult(self, x, y)
    Mult = _swig_new_instance_method(_matrix.AbstractSparseMatrix_Mult)

    def AddMult(self, x, y, val=1.):
        r"""AddMult(AbstractSparseMatrix self, Vector x, Vector y, double const val=1.)"""
        return _matrix.AbstractSparseMatrix_AddMult(self, x, y, val)
    AddMult = _swig_new_instance_method(_matrix.AbstractSparseMatrix_AddMult)

    def MultTranspose(self, x, y):
        r"""MultTranspose(AbstractSparseMatrix self, Vector x, Vector y)"""
        return _matrix.AbstractSparseMatrix_MultTranspose(self, x, y)
    MultTranspose = _swig_new_instance_method(_matrix.AbstractSparseMatrix_MultTranspose)

    def AddMultTranspose(self, x, y, val=1.):
        r"""AddMultTranspose(AbstractSparseMatrix self, Vector x, Vector y, double const val=1.)"""
        return _matrix.AbstractSparseMatrix_AddMultTranspose(self, x, y, val)
    AddMultTranspose = _swig_new_instance_method(_matrix.AbstractSparseMatrix_AddMultTranspose)
    __swig_destroy__ = _matrix.delete_AbstractSparseMatrix

# Register AbstractSparseMatrix in _matrix:
_matrix.AbstractSparseMatrix_swigregister(AbstractSparseMatrix)



